// my_function_template.cpp

#include "my_essentials.hpp"

// Template function for swapping values:
template<typename T> 
// this notation with keyword template makes the 
// input parameter T as the type parameter and 
// this notation can also be written as 
// template<class T>, because, in general, 
// the keyword typename can be replaced with 
// the name class.

void swap_values(T& a, T& b){ // The &-sign says the
// input parameters are passed-by-reference, 
// the details are in section 2.2.
	T temp = a;
	a = b;
	b = temp;
}

// Application of the template function 
// for swapping values:

class class1{
public:
	std::string result1(){
		return(" word_from_class1 ");
	}
};

class class2{
public:
	std::string result2(){
		return(" word_from_class2 ");
	}
};

class abc{
public:
	double a, b, c;
};

int main()
{
	abc abc_one, abc_two;
	abc_one.a = 1;
	abc_one.b = 2;
	abc_one.c = 3;
	abc_two.a = 4;
	abc_two.b = 5;
	abc_two.c = 6;
	cout<< "\nswap_values(class abc::abc_one,"
		<<"    class abc::abc_two):";

	cout << "\nbefore swap:\tabc_one: "
	<<abc_one.a<<";"<<abc_one.b<<"; "<<abc_one.c;
	cout << "\t\tabc_two: "
	<<abc_two.a<<"; "<<abc_two.b<<";"<<abc_two.c
	<<endl;
	swap_values(abc_one, abc_two);  // It swaps 	
	// the classes abc_one and abc_two.
	cout << "after swap:\tabc_one: " 
	<<abc_one.a<<"; "<<abc_one.b
	<<";"	<<abc_one.c;
	cout << "\t\tabc_two: "
	<<abc_two.a<<"; "<<abc_two.b
	<<"; "<<abc_two.c<<endl;
	class1 cl1;
	class2 cl2;
	// swap_values(cl1, cl2);  ? this statement
	// has an error, because cl1 and cl2 do not 	
	// belong to the same class. But their 
	// member functions, result1 and result2, 
	// are of type string, so those can be 
	// used in this template.

	string st1 = cl1.result1(), 
				st2 = cl2.result2();
	cout 
	<< "\nswap_values(string class1::result1(),"
	<<" string class2::result2()): ";
	cout 
	<< "\nbefore swap:\t" << st1 << "\t" 
	<< st2 <<endl;
	swap_values(st1, st2); 
	//swaps the strings st1 and st2.
	cout 
	<< "after sawp:\t"<<st1<<"\t"<<st2<<endl;
	double n1 = 100, n2= 200;
	cout 
	<< "\nswap_values(double n1, double n2):"
	<<"\nbefore swap:\t"<<n1<<"\t"<<n2<<endl;
	swap_values(n1,n2); 
	// swaps the doubles n1 and  n2.
	cout 
	<<"after swap:\t" << n1 << "\t" << n2 <<endl;
	just_pause();
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
