// my_class_template.cpp
#include "my_essentials.hpp"

template<typename T, 
	typename Allocator = allocator<T>> /* 
	By default, C++ provides effective 
	memeory allocators to its containers 
	for memory management; however 
	programmers are also allowed 
	to provide custom memory allocators 
	for their specific requirement.*/
class vector_demo{
	int sz; // Size of the vector_demo.
	T* elem; // A * followed by an object name 
	// is called a pointer and it points
	// to the first element of the object 
	// named elem of type T.
public:
	vector_demo() :sz(0), elem(0) {}
	// It is called the default constructor,
	
	vector_demo(int s); 	
	// it is also a constructor 
	// and here it allocates s number of objects
 	// of type T.
	
	vector_demo(const vector_demo&);  
	// It is called
	// copy constructor.

	~vector_demo() {delete[] elem;}  
	// It is called a destructor, because it 
	// deletes the pointers with the command
 	// delete[].

	int size() const {return sz;} 
	// It returns the present size of 	
	// the vector_demo.
	
	// The following are declarations of a
	// few member functions which are useful 
	// for adding elements to the vector_demo.
	void resize(int newsize);
	void push_back(const T& d);
	void reserve(int newalloc);
};

// As the template vector_demo is for structural 
// demonstration, it did not go into technical
// levels (such as smart pointers and iterators, 
// which are discussed in Chapter 2 Pointers+).

//The following sample problem uses 
// the class template <vector> of 
// the Standard Template Library (STL).  

#include <vector>

// Introducing the STL container vector 
// in the personal budget development.

class annual_budget{
		public:
		int year;
		double income;
		double expense;
		double investment;
	};


int main(){	
	annual_budget data; // "data" is the object 
		// of type "annual_budget".
	vector<annual_budget> ab; // "ab" is a vector
 		// of type "annual_budget".
	ab.reserve(10); // it reserves 
		// memory allocation for (10) 
		// objects of type annual_budget;
	cout 
	<< "\n Initial size of the vector 'ab' = " 
	<< ab.size() << endl;
	//Let
	data.year = 2011; 
	data.income = 10000; 
	data.expense = 9000; 
	data.investment = data.income - data.expense;
	ab.push_back(data); 	// "data" is assigned to
 		// the above shown values and added
		//at the end of "ab".
	data.year = 2012; data.income = 10100; 	data.expense = 9200; 
	data.investment = data.income - data.expense;
	ab.push_back(data);
	cout << "\n Final size of the vector 'ab' = "
	<< ab.size()<< endl;
	
	just_pause();
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
