// auto_and_decltype.cpp
#include "my_essentials.hpp"

// Part (a) comparing auto and decltype:

void compare_auto_and_decltype()
{	
	auto x = 7; 

	auto y = 3;

	auto i = x+y; // The auto finds the type of 
	// (x+y) as integer, so 'i' is declared as an 
	// integer and assigned the 'i' with the 
	// resulted expression value of 10.

	decltype(x+y) j = x+y; // The decltype(x+y) 
	// yields the type of (x+y) as integer, 
	// so 'j' is declared as an integer and 
	// assigned the 'j' with the resulted 
	// expression value of 10. 
	
	if(i==j)
		cout << "\n i = " << i 
		<< "; j = " << j 
		<< "\n Both auto and decltype"
		<< " identified the type of"
		<< "\n the given expression"
		<< " as an integer!\n";
}
//----------------------------------

// Part (b) Declaring template functions:

// The direct method: 
template<typename T>
T divide1(T num, T den)
{
	return num/den;
}
//------------------------

/* The method of using both decltype and auto:
  
   $$ 	The following function template involves 
   	a process called "perfect forwarding", 
	because it uses a forwarding functions wrap 
	called std::forward<> to resolve lvalue (T&) 
	and rvalue (T&&) references. The lvalue and
	rvalue references are discussed in section 
	2.4 lvalue, rvalue and universal references.
	The perfect forwarding is explained in 
	Scott Meyers Overview of New C++ (C++11).
  
   $$ 	This method is very useful for 
      	generic programmers.
*/

template<typename T>
auto divide2(T&& num, T&& den) // The && stands for 
	// universal reference, refer section 2.4.
	-> decltype(forward<T>(num)/forward<T>(den)) 
	// The symbol -> is known as a 
	// trailing return type and it is 
	// an explicit way of asking for a 
	// return type.
{
	return forward<T>(num)/forward<T>(den);
}
//-------------------------------------

int main()
{
	compare_auto_and_decltype();
	auto x = 10, y = 2;
	cout	<< "\n Division of " 
		<< x << " by " << y 
		<< " using:"
		<< "\n (a) direct declaration = " 
		<< divide1(x, y)
		<< ";\n (b) both auto and decltype = " 
		<< divide2(x, y)
		<< ".\n";
	just_pause();
}
//---------------------------------------

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
