// efficiency_of_reference.cpp

#include "my_essentials.hpp"
#include <numeric> 
/*<numeric> provides the Standard Library function 
for operations on a numerical sequence and we will 
add this header in our my_essentials.hpp */

class reference_property2{

double sum_by_value(vector<double> vec){
	return(accumulate(vec.begin(),vec.end(),0));
/* 
Where, begin() and end() are member functions of 
the STL vector and are called iterators. 
The begin() points to the first element of 
the vector and end() points to one after 
its last element.  The second figure 
in section 2.15 STL iterators  illustrates 
this procedure.

The function: accumulate with parameter list: 
	(a point to start in a sequence for summing, 
	  a point to stop, 
		additional value for summing)
 is provided by the <numeric> */
}

double sum_by_reference(vector<double> &vec){ 
	return(accumulate(vec.begin(), vec.end(),0));
}

double sum_by_const_reference
		(const vector<double>&vec) {
	return(accumulate(vec.begin(), vec.end(),0));
}

public:
	void test();
};
	
void reference_property2::test()
{
	vector<double> num(1000000, 5); /* 	vector num 	
	declared and initialized with millions of fives */

	double total =0;
	cout << "\n Summing up a trillion numbers:";
	
	cout << "\n\n\t(a) Pass-by-Value:"
		 << "\n\t\t...processing....\n";
	for(double i =0; i != 1000; ++i)
	total += sum_by_value(num); 
			//? This took 505 seconds
	cout << "\n\t\tTotal = " << total << endl; 
	total =0;
	
	cout << "\n\n\t(b) Pass-by-Reference:"
		 << "\n\t\t...processing....\n";
	for(double i =0; i != 1000; ++i)
	total += sum_by_reference(num); 
					//? 12 seconds 
	cout <<  "\n\t\tTotal = " << total << endl;
	total =0;
	
	cout << "\n\n\t(c) Pass-by-Const-Reference:"
		 << "\n\t\t...processing....\n";
	for(double i =0; i != 1000; ++i)
	total += sum_by_const_reference(num);
					 //? 12 seconds
	cout << "\n\t\tTotal = " << total << endl;	
}

int main()
{
	reference_property2 rp2;
	rp2.test();
	just_pause();
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
