// lvalue_rvalue_universal_references.cpp
#include "..\..\my_essentials.hpp"

void references_demo(int r)
{		
	// (1) lvalue-reference:
	auto& t = r;	// fine, 
	// the lvalue-reference can be assigned 
	// to the defined object(lvalue)
	
	// auto& u = 10; // error, 
	// the lvalue-reference cannot be assigned 
	// to a temporary object (rvalue)

	//------------------------------------
	
	// (2) rvalue-reference:
	const auto& v = 10; // fine
	// this constant reference can be assigned 
	// to a temporary object (rvalue), hence it
	// works as a rvalue-reference

	int&& w = 10; // fine
	// this double '&' with a type name generates
	// rvalue-reference for a temporary object.

	//---------------------------------------

	// (3) universal-reference:
	// if a reference has && and its type is 	
	// identified by auto or template typename T, 
	// then it can be called as 
	// an auto-reference. 
	auto&& x = 10;	// fine, 
	// the universal-reference is assigned 
	// to a temporary, so it works 
	// as a rvalue-reference

	auto&& y = r;	// fine, 
	// the universal-reference is assigned 
	// to the lvalue, so it works 
	// as a lvalue-reference 

	// int&& z = r; // error, 
	// if auto is not used 
	// then a rvalue-reference cannot be assigned 
	// to lvalue-reference 

	//---------------------------------------

	cout << "r = " << r << endl;
	cout << "t = " << t << endl;
	// cout << "u = " << u << endl;
	cout << "v = " << v << endl;
	cout << "w = " << w << endl;
	cout << "x = " << x << endl;
	cout << "y = " << y << endl;
	// cout << "z = " << z << endl;
}

int main()
{
	auto r = 10;	// the object(lvalue) 'r' 
	// is constructed with value 10;
	references_demo(r);
	just_pause();
}
