// pointer_arithmetic.cpp

#include "my_essentials.hpp"

int main()
{
	//  Expressing Array and Vectors in Pointers 
	//  - an application of pointer arithmetic.

	// Let us define an array of prime numbers:
	double ary[]  = {2, 3, 5, 7};

	// Let us define a pointer for this array
	double *ptr = &ary[0];
	// so, the pointer ptr set to point at 
	// the first element.

	cout 	<< "\n Expression of the array"
		<<  " in terms of pointers: ";
	for(int i =0; i != 4; ++i, ++ptr) 
	// ++ moves up in address ptr by one unit.
	{
		cout 
		<< "\n\tary["<< i<< "] = " << ary[i]
		<< "\t*ptr = "<< *ptr << 
		"\t*(ary + "<<i<<") = "<< *(ary + i);	
	}
	
	// Let us make a vector using the array
	vector<double> vary(ary,ary+4);

	// Let us define a pointer for this vector 
	// using one of its(STL Containers) member
	// type named "pointer"
	vector<double>::pointer vptr = &vary[0];
	// so, the pointer vptr is set to 
	// point at the first element
	
	cout 	<< "\n\n Expression of the vector" 
		<< " in terms of pointers: ";
	
	for(int i =0; i != 4; ++i, ++vptr) //  
	// ++ moves up in address vptr by one unit.
	{
		cout 	<< "\n\tvary[" << i << "] = "
		<< vary[i] << "\t*vptr = "<< *vptr;
	}
	
	cout	<< "\n\n Beauty of movement of" 
		<< " the address in pointers: ";
	
	for(int i =0; i != 4; ++i, --ptr, --vptr) // 	    
		// -- moves down in address by one unit.
	{
		cout 	<< "\n\tptr = " << ptr 
			<<"\tvptr = "<< vptr;
	}
	
	cout 	<< "\n These addresses generally" 
		<< " differ by 8 bits,\n because" 
		<< " the size of double is 8 bits!" 
		<< endl;
	
	just_pause();
	return 0;
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
