//funcPtr_function.cpp

#include "my_essentials.hpp"
//Defining individual functions:
double average(const vector<double> &);
double add(double a, double b) {return a+b;}
double sub(double a, double b) {return a-b;}
double mul(double a, double b) {return a*b;}
double divi(double a, double b) {return a/b;} 
/* We are naming it divi, instead of div, because the Standard Library has a function with the name div */

//Declaring old style function pointers:
//Single Dereferencing:
double (*single_maths) (const vector<double>&) = average;

//Multiple Dereferencing:
double mutiple_maths (double a, double b, 
	double (* my_mat) (double, double) ) {
   	return (* my_mat)(a,b);
}	
// The pointer *my_maths dereferences to add, sub,
// mul, and divi whose arguments are the same, 
// i.e.,  (double, double), so it cannot dereference
// to the function: average(const vector<double>&)

int main()
{

	cout <<"\nUSING OLD STYLE "
			<< "\"function pointer\":"; 
	cout << "\nSingle Dereferencing:\n";
	vector<double> year_days;
	year_days.reserve(4);
	//Let us assign the days of 4 consecutive years:
	year_days.push_back(365);
	year_days.push_back(365);
	year_days.push_back(366);
	year_days.push_back(365);
	/* 
	Then to calculate the average number of days 	per year the function pointer single_maths 	is used because it points to the function 	average 
	*/
	cout 
		<< "\nsingle_maths(year_days) = " 
		<< single_maths(year_days)<< endl;

	cout<<"\nMultiple Dereferencing:\n";
	//Let
	double a = 10.0, b = 20.0;
	
	cout << "\nmutiple_maths("
			<<a<<", "<< b <<",  add) = " 
			<< mutiple_maths(a, b, add); 

	cout << "\n\nmutiple_maths("
			<<a<<", "<< b <<",  sub) = " 
			<< mutiple_maths(a, b, sub); 

	cout << "\n\nmutiple_maths("
			<<a<<", "<< b <<",  mul) = " 
			<< mutiple_maths(a, b, mul); 

	cout << "\n\nmutiple_maths("
			<<a<<", "<< b <<",  divi) = " 
			<< mutiple_maths(a, b, divi)
			<< endl;

	cout << "\n\nUSING NEW STYLE "
				<< "\"function<>\":";
	function<double(double,double)> my_maths;

	my_maths = add;
	cout << "\n When my_maths = add" <<endl
		<<"\tmy_maths(" << a <<", "<< b 
		<< ") = " << my_maths(a, b);

	
	my_maths = sub;
	cout << "\n\n When my_maths = sub" <<endl
		<<"\tmy_maths(" << a <<", "<< b 			
		<< ") = " << my_maths(a, b);

	my_maths = mul;
	cout << "\n\n When my_maths = mul" <<endl
		<<"\tmy_maths(" << a <<", "<< b 			
		<< ") = " << my_maths(a, b);

	my_maths = divi;
	cout << "\n\n When my_maths = divi" <<endl
		<<"\tmy_maths(" << a <<", "<< b 
		<< ") = " << my_maths(a, b);


	cout << "\n\nSince my_maths is declared as a"
		<<"two parameters function with "
		<<"return type double, i.e., "
		<<"<double(double, double)>, "
		<<"it cannot be "
		<<"assigned to the function "
		<<"average, which is " 		
		<< "<double(vector<double>)>.\n"
		<<"Therefore, we declare another"
		<<" function, such as\n"
		<<"function<double(vector<double>)> "
		<< "my_maths_average;\n";
	
	function<double(vector<double>)> 
					my_maths_average;

	my_maths_average = average;
	cout << "\n When my_maths_average = "
		<<"average" <<endl
		<<"\tmy_maths_average(year_days) = " 
		<< my_maths_average(year_days) <<endl;

	cout << "\nConclusion:\n\tThus, the new style"
		<< " function<> can replace the old"
		<< " style function pointer.\n";
	just_pause();
}	

double average(const vector<double> &vec)
{
	return ( 
	(accumulate(vec.begin(), vec.end(),0))
		/static_cast<double>(vec.size()) 
	);
/* 	
The numerator (the Standard Library algorithm: accumulate()) 
is of type double, but the denominator vec.size() is of type int, 
so, its type has been converted to double by static cast 
[refer to appendix section C.3 for a description about casting]. 
*/
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
