//memory_storage.cpp
//Points to be noted from this code:
// (1) pointer dereferencing and memory management;
// (2) three types of storage classes; and
// (3) member access operators.

#include "my_essentials.hpp"

const unsigned int data_size = 1024; /* This is a global variable, so it is in Static Storage.
This global variable is available to all the objects that follow, i.e., class unit_data_sort, function Bootstrap() and the main() function. */

class unit_data_sort{
public:
double* sort_data(double* data)
{ // The pointer data 
  //is borrowed from the function Bootstrap(), 
  //so it is its responsibilty to delete it.
  return sort_verify(main_sort(pre_sort(data)));
}
private:
double* pre_sort(double* pre_sort_data){
	//....;
	return(pre_sort_data);
} 
double* main_sort(double* main_sort_data){
	//....;
	return(main_sort_data);
}
	double* sort_verify(double* sort_verify_data){
		//....;
	return(sort_verify_data);
}

};

class read_write_data{ /* All pointers are borrowed from the function Bootstrap(), so no free storage memory is created in this class. */
public:
void read_data(string* location, double* data_set){
cout<<"\nreading the data_set from the location";
}
	
void write_data(string* location, double* data_set){
cout<<"\nwriting the data_set into the location\n";
}

};

int Bootstrap()
{
	unit_data_sort* uds	= new unit_data_sort; 
	string* filename	= new string; 
	double* data		= new double(data_size);
	// These pointers are placed in free storage.
	
	unsigned int size_of_file_list = 100; 
//This variable is placed in the automatic storage.

for(unsigned int i =0; i !=  size_of_file_list; ++i){ 
	cout << "\n\tData Set: "<<i+1; 
	read_write_data rw;
	*filename = "input file location and name";
	rw.read_data(filename, data);
	data = uds->sort_data(data); 
/* In this statement, please note the following two points:
 
(1)	Left-side data is cleaner output and the right side is the raw data,

(2)	As the member function of a class is mentioned by '.'(dot operator), 
the member function of a  class through the pointer is mentioned either 
by "->"(arrow operator) or in this way: (*uds).data_size; both of 
these operators ( "." and "->") are called member access operators.
*/
*filename = "output file location and name";
rw.write_data(filename, data);
if((i+1)%10==0)just_pause(); 
// Pauses at every 10th data.
}

	delete [] data; 
/* So no memory leak due to the pointer data. However, 
	if any of the program components crash/break the flow of the program execution, 
	then the pointer cannot be deleted and that causes memory leak. 
	This subject is discussed in Chapter 6.
 
	Also note that the destruction of the pointer array should be with 
	an array symbol "[]", whereas destruction of a single object needs 
	to use just delete without an array symbol "[]". */
	delete uds; 
	delete filename; 
	cout<<"bootstrap ends"<<endl;just_pause();
	return 0;
}

int main()
{
	Bootstrap();
	cout<<"main ends"<<endl;
	just_pause();
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
