//smart_pointers.cpp
// Caution: This code compiles 
// in gcc 4.7 and in VC12, but not in VC10.

#include "my_essentials.hpp"

void smart_pointers();

int main()
{
	smart_pointers();
	just_pause();
}

#include <memory> 
// Smart pointers are defined in <memory>

void smart_pointers()
{
	vector<double> data(5);

	for(auto &i: data) 
	{
		static double j = 1;
		i = j*j;
		++j;
	}
	
	cout << "\nLet us have a vector:\n"
	<< "\tdata\t= (";
	for(auto element:data)
		cout << element << ',';
cout	<< ");\n\nThen, this data is copied into a new unique pointer (uq)," 
		<< "then this is moved to another unique pointer:\n\tuq_mv\t= (";

	// UNIQUE POINTERS:
unique_ptr<vector<double>> 
			uq(new vector<double> (data));
// unique_ptr<vector<double>> uq_copy(uq);
// this copy is not allowed from one unique_ptr 
// to another unique_ptr
unique_ptr<vector<double>> uq_mv(move(uq));//but transfer is allowed by using the keyword: move 

	for(double element:*uq_mv)
		cout << element << ',';
	cout <<");"<<endl;

cout	<< "\nA shared pointer can perform both copy and move.\n"
		<< "So, the unique pointer uq_mv has been copied to a "
		<< "new shared pointer (sh), then that is moved to "
		<< "another shared pointer sh_mv, then that is copied "
		<< "to another shared pointer:\n\tsh_copy\t= (";
	
// SHARED POINTERS:
// shared_ptr<vector<double>> 
//	sh(new vector<double> (*uq));//runtime 
// error, because uq has already moved to uq_mv
shared_ptr<vector<double>> 
	sh(new vector<double> (*uq_mv));//fine
shared_ptr<vector<double>> 
	sh_mv(move(sh));//move is allowed
shared_ptr<vector<double>> 
	sh_copy(sh_mv);//copy is allowed
	
	for(double element:*sh_copy)
		cout << element << ',';
	cout <<");"<<endl;

	

cout	<< "\nSmart pointers can be used in a container as pointers to "
		<< "huge amounts of data, thus the elements of the vector of "
		<< "shared pointers of the vector of double are:\n\tvec_of_sh =\t";
	
	// CONTAINER OF SHARED POINTER:
	vector<shared_ptr<vector<double>>> vec_of_sh;
	
	vec_of_sh.push_back(sh_copy);
	
	vector<double>data_another;
	data_another.push_back(90);
	data_another.push_back(180);
shared_ptr<vector<double>> 
      sh_another(new vector<double> (data_another));
	vec_of_sh.push_back(sh_another);
	
	for(auto ele:vec_of_sh)
	{
		cout << '(';
		for(auto element:*ele)
		{
			cout << element << ',';
		}
		cout <<"); ";
	}
	
	// CONTAINER OF UNIQUE POINTER:

	vector<unique_ptr<vector<double>>> vec_of_uq;
		
unique_ptr<vector<double>> 
		uq_one(new vector<double> (data));
	vec_of_uq.push_back(move(uq_one));//please note, 'move' 
				// must be involved in the push_back to 
				// the vector of unique pointers.
	
unique_ptr<vector<double>> 
      uq_another(new vector<double> (data_another));
	vec_of_uq.push_back(move(uq_another));

cout	<< "\n\nThe same thing can be done using the unique pointer, "
		<< "but with \"move semantics\". Thus, the elements of "
		<< "the vector of unique pointers of the vector of double is:"
		<< "\n\tvec_of_uq =\t";
	
// a range-based for loop cannot work with the 
// vector of unique pointers, so we use 
// the traditional method as follows:
	unique_ptr<vector<double>> uq_temp; 
	for(unsigned int i = 0; 
				i <vec_of_uq.size(); ++i)
	{
		cout << '(';
		uq_temp = move(vec_of_uq[i]);
		for(double element:*uq_temp)
			cout << element << ',';
		cout <<"); ";
	}

// WEAK POINTER:
cout	<<"\n\nA weak pointer(wk) can be constructed from a shared pointer(sh1)"
		<< "and it can then construct another shared pointer(sh2) by using its " 
		<< "function wk.lock(). Thus, when sh1 and sh2 are active wk.lock() "
		<< "does not show NULL. So, we get:\n";  
	
	shared_ptr<vector<double>> 
			sh1(new vector<double> (data));
	
	weak_ptr<vector<double>> 
		    wk(sh1); //wk constructed from sh1
	
	shared_ptr<vector<double>> sh2(wk.lock());
// weak pointer copies content of sh1 to sh2 via the 
// weak pointer function wk.lock()
	
	if(wk.lock()) 
		cout << "\n\"The object is busy with shared pointers\"\n";
	else
		cout << "\n\"The shared pointers have finished their job!\"\n";
	
	// shared pointers are released:
	sh1.reset();
	sh2.reset();

	cout	<< "\n\n Once shared pointers are released from their object, "
			<< "wk.lock() produces NULL. So we get:\n";
	
	if(wk.lock()) 
		cout << "\n\"The object is busy with shared pointers\"\n";
	else
		cout << "\n\"The shared pointers have finished their job!\"\n";
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
