//iterators.cpp
#include "my_essentials.hpp"

void illustrate_iterators()
{
int arr[] = {1111, 1134, 1134, 1124, 1124, 1134, 1111, 1176, 1124, 1134, 1176};
vector<int>data(arr, arr+ sizeof(arr)/sizeof(arr[0]) );
//vector<int>data2{1111, 1134, 1134, 1124, 1124, 1134, 1111, 1176, 1124, 1134, 1176}; //fine with gcc4.7.0
cout<< "\n Illustration of iterators: \n"
	<< "\n Given vector: data =\n\t( ";
for(auto element: data) // 'the range-based for' do not work with VC10.
	cout << element <<"; "; 

cout<<")\n\n Let us define an iterator, say *it for this vector.\n"; 
vector<int>::iterator it = data.begin();

/*
Note: 
In STL containers, each container template class defines its own iterator member class, so that vector<int>::iterator is different from the list <int>::iterator.
*/
cout << " Thus,\t\t*it = " << *it <<" that is data[0]\n Iterators can be incremented, i.e, ++it;\n"; 
++it; 
cout << " After ++\t*it = " << *it <<" that is data[1]\n Therefore, iterator *it dereferences through vector data.\n So, let us express the entire vector using the iterator:  \n\t( ";
for(auto it = data.begin(); it!=data.end(); ++it)
	cout << *it <<"; ";
cout<<")\n";

cout << "\n Task: Since, this vector has repeated numbers,\n Can we keep only one copy of each number and remove the duplicates?\n Yes, we can!\n For that, we pass the iterator through three type of STL algorithms as shown below:\n";

cout <<"\n Step.1 Sort in ascending order:\n";

sort(data.begin(), data.end());

cout<<" After the iterator has passed through sort(): \n\t( ";
for(auto element: data)
	cout << element <<"; "; 
cout<<")\n\n Step.2 Move the adjacent repeated elements to the left hand side:\n";

auto my_unique = unique(data.begin(), data.end());

cout<<" After the iterator has passed through unique(): \n\t( ";
for(auto element: data)
	cout << element <<"; "; 
/*
The function unique() has returned an iterator to the identifier my_unique, starting from which the repeated elements are pushed up to one before the end(). 
NOTE: THE STL ALGORITHMS CAN DISPLACE THE ELEMENTS BUT IT CANNOT ERASE THEM, HOWEVER EACH STL CONTAINER TEMPLATE CLASS HAS A MEMBER FUNCTION CALLED erase() OPERATION FOR ERASING THE SPECIFIED ELEMENTS.  
Therefore, to remove the duplicates we apply the operation erase() from the iterator my_unique till the end().
*/ 
cout<<")\n\n Step.3 Erase out those repeated elements:\n";

data.erase(my_unique, data.end());

cout<<" After the iterator has passed through erase() i.e., \n Final:\t( ";
for(auto element: data)
	cout << element <<"; "; 
cout<<")\n";
}

int main()
{
 illustrate_iterators();
 just_pause();
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
