// operator_overloading.cpp
#include "my_essentials.hpp"

class fruit_basket{
public:
	// Default constructor 
	fruit_basket():apple(0), pear(0){}
	
	// Constructor from arguments
	fruit_basket(double a, double b)
			:apple(a), pear(b){}
	/*	
	Note:
		Constructors are explored 
		in the following Chapter. 
	
	(1) OPERATOR+ OVERLOADING for
		counting fruits in baskets
	*/
	fruit_basket operator+
			(const fruit_basket basket) 
	{	
		apple +=basket.apple;
		pear +=basket.pear;
		return *this;
	}
	
	/*
	(2) OPERATOR<< OVERLOADING for 
		printing basket list. 
	*/
	friend ostream& operator<<(ostream& os,
			const fruit_basket&  basket)
	{
		os	<<"("<<basket.apple   
			<<" apples; " <<basket.pear 
			<<" pears)\n";
		return os;
	} /* 
	Here we used "ostream" from the Standard Library.
	Since it is not a member function of the class Team, 
	but its external function, 	it is declared as 
	the "friend" function.
	This is so because, if an external function declared as 
	a friend to a class, then that function can access 
	all the members of that class. 
*/

private:
	double apple, pear;
};

 
int main()
{
	const fruit_basket A(4,4), B(5,2), C(3,6);
	fruit_basket sum(0,0);
	
	sum = sum + A + B + C; 

	/* 
	The summing using the operator+ requires that 
	the value to the immediate right of the equals sign 
	should be a non-constant, therefore this pattern of 
	summing is used rather than sum = A + B + C, 
	which would result in an error because the 
	constant value of A could not be changed to 
	the sum of the values of A and B.
	*/
	
	cout<<"\n Fruit list, basket-wise:\n"
		<<"\n A: "<<A
		<<"\n B: "<<B
		<<"\n C: "<<C 
		<<"\n Total number of fruits: "<<sum;
	just_pause(); 
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
