// lambda_fn_for_sine_wave.cpp 

#include "my_essentials.hpp"

int Lambda() 
{

const double radian_per_degree = (22.0/7.0)/180.0;
auto radian = [radian_per_degree](double degree)
	->double{return degree*radian_per_degree;};
/* 
where, "->double" asks for double as the return type. 
This is known as a trailing return type and 
it is an explicit way of asking for a return type.
*/

vector<double> data(361);
int i =0;
/* 
To evaluate the sequence of data points in 
the vector data, let us incorporate 
a sequence algorithm of STL, of the format:
for_each(beginning point, ending point, 
calling functor/lambda-expression for each point); 
*/
for_each(
data.begin(), data.end(), 
	[radian, i](double &data_event) mutable
	{
		data_event = sin(radian(i));
		++i;
	}
);
/* 
where, the function for_each() represents 
the variable data_event inside the lambda function 
to each point of the container 
for assigning corresponding sine value.

In the lambda function, the captured parameters of 
pass-by-value behave as constant parameters, 
however to make those change their values within 
the lambda function the keyword mutable is used. 
But pass-by-reference does not behave as 
a constant reference unless that original parameter 
outside the lambda function was declared as a constant. 
In the above lambda-expression, the value of 
the variable i is zero before and after 
the lambda expression, but within the expression, 
it changes from 0 to 360, because it is made mutable 
by the keyword mutable.
*/
for(unsigned short i =0;i != data.size(); ++i)
	cout<<"\t"<<i<<"\t"<<data[i]<<endl;
   just_pause();
   return 0;
}

int main()
{
	Lambda();
}
