//algorithms+.cpp

#include "my_essentials.hpp"

template<typename T>
void demo_algorithms(vector<T>, vector<T>);

int main()
{
	int arr1[] = {2,7,4,1,6,9,8,100,3,5};
	vector<int> vec1
	    (arr1,arr1+sizeof(arr1)/sizeof(arr1[0])); 
	//If you are using gcc 4.7.0 then	
	//you can do direct assignment:
	//vector<int> vec1{2,4,3,5,2,5,1,6,-2,7,2,9};
	int arr2[] = {5,6,7,8,9,10,11,12,13,14,15};
	vector<int> vec2
	    (arr2,arr2+sizeof(arr2)/sizeof(arr2[0])); 
	demo_algorithms(vec1,vec2);
	just_pause();
}

template<typename T>
void demo_algorithms(vector<T>vec1, vector<T>vec2)
{
	vector<T>vec_out1, vec_out2, vec_out3;
	cout_container(" Let array vec1:\n\t", vec1);
	
	auto b1 = vec1.begin();
	auto e1 = vec1.end();

	cout<< endl;
cout<<"1. Format: algorithm(b1,e1):\n";
	sort (b1,e1); 
	//By default, i.e., without any predicate or 
	//function, the sort() will sort the 
	//container in ascending order.
	cout_container("\n sort (b1,e1):\n\t", vec1);
	cout<<"................................\n\n";

cout<<"2. Format: algorithm(b1,b1+N,e1):\n";
	if(b1+3<e1)
	{
	  rotate(b1,b1+2,e1);
	  cout_container
		("\n rotate(b1,b1+2,e1):\n\t", vec1);
	}
	cout<<"................................\n\n";

cout<<"3. Format: algorithm(b1,e1,f):\n";
	cout << "\n for_each(b1,e1,f)  couts:\n\t(";
	for_each(b1,e1,[](T &x){cout << x<<' ';});
	cout << ")\n";
	cout<<"................................\n\n";

cout<<"4. Format: algorithm(b1,e1,v):\n";
	cout << "\n find(b1,e1,v):\n";

auto findx = [=](int v)->int
{
	auto ptr = find(b1,e1,v);
	cout << "\n the value v = "<< v;
	if(ptr== e1)
	{
		cout << " does not exist.\n";
		return 0;
	}
	if((ptr>b1)&&(ptr<e1-1))
		cout << " exists between " 
		<< *(ptr-1) << " and " << *(ptr+1);
	else
		if(ptr == b1)
			cout << " is the first element";
		else
			cout << " is the last element";
	cout << endl; 
	return 0;
};
	
	findx(0);
	findx(6);
	findx(10);
	findx(11);
	cout<<"................................\n\n";
	
cout<<"5. Format: algorithm(b1,e1,v,f):\n\n"; 
	cout<< "the multiplication product of\n"
		<< " all the elements in vec1:\n"
		<< " accumulate(b1,e1,1,multiplies<int>())\n = " 
		<< accumulate(b1,e1,1,multiplies<T>())
		<< endl;
	cout<<"................................\n\n";

cout<<"6. Format: algorithm(b1,e1,v1,v2):\n";
	replace(b1,e1,100,10);
	sort(b1,e1);
	cout<< "\n Replace 100 with 10\n"
		<<" replace(b1,e1,100,10):\n"
		<<"\n and then sort() makes";
	cout_container
		( " vec1: \n\t",vec1);
	cout<<"................................\n\n";
	
cout<<"7. Format: algorithm(b1,e1,out):\n";
	// Algorithm: copy():
	// the copy algorithm just copies and 
	// it cannot construct an object.
	// So make sure you have 
	// enough space available in vec2 
	// for the contents of vec1, such as:
	if(vec_out1.size() >= vec1.size())
		copy(b1,e1,vec_out1.begin());		
	else
		copy
		(b1, e1, back_inserter(vec_out1) );
	cout<<" copy(b1,e1,back_inserter(vec_out1))";
	cout_container(
	"\n vec1 copied to vec_out1:\n\t",
		vec_out1);

	cout_container(
	"\n Let us read the next input array vec2: 	\n\t", vec2);

	auto b2 = vec2.begin();
	auto e2 = vec2.end();
	cout<<"................................\n\n";

cout<<"8. Format: algorithm(b1,e1,b2,e2):\n";
	auto flag 
	= lexicographical_compare(b2,e2,b1,e1); 
	cout<<
	"\n lexicographical_compare(b2,e2,b1,e1) \n"
	<< "\n i.e., Lexicographically,\n\t"
	<< " Is vec2 < vec1 ?: ";
	if(flag)
		cout << "Yes!" << endl;
	else
		cout << "No!" << endl;
	cout<<"................................\n\n";

cout<<"9. Format: algorithm(b1,e1,b2,e2,out):\n"; 
	merge(b1,e1-(e1-b1)/2,
		b2+(e2-b2)/2,e2,
		back_inserter(vec_out2));

	cout<<"\n Merge first half of vec1\n"
		<<" with second half of vec2:";
	cout_container("\n vec_out2:\n\t", vec_out2);
	cout<<"................................\n\n";

cout<<"10. Format: algorithm(b1,e1,b2,e2,out,f):\n"; 
	merge(b1,e1,b2,e2,
		back_inserter(vec_out3), less<T>());

	cout<<"\n Merge vec1 with vec2, "
		<<" but in asecending order:";
	cout_container("\n vec_out3:\n\t", vec_out3);
	cout<<"................................\n\n";
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
