// encapsulation_in_inheritance.cpp

# include "my_essentials.hpp"

class flight_engine {};

class flight_pilot {};

class flight_minimum_fare {};

class flight_office_record:
	private flight_engine,
	protected flight_pilot,
	public flight_minimum_fare
{};

class flight_fare:
	private flight_office_record
{ /* This class can access flight_pilot 
	 and flight_minimum_fare, but 
     it cannot access flight_engine. */ 
public:
	double total_fare();
};

class ticket_payment:
	public flight_fare
{}; 
/* This class cannot access flight_engine, 
   flight_pilot and flight_minimum_fare, 
   because flight_office_record 
   is inaccessible (private in flight_fare). 
   So, it can only access the public member 
   total_fare() in flight_fare().  */

int main()
{}

/*
In plain words, 

	A public inheritance means 
	the derived class is an object 
	of type base class;

	A private inheritance means 
	the derived class uses 
	an object of type base class 
	for its implementation that is 
	similar to composition; however, 
	composition is preferable 
	to private inheritance.

	A protected inheritance is 
	very rare and it means it is like 
	a public inheritance for 
	its derived class and inaccessible 
	to all other objects. 

	Apart from these non-virtual 
	inheritances, C++ defines 
	virtual inheritance too. 
	The use of virtual inheritance 
	is inevitable in scenarios of 
	multiple inheritances. However, 
	an object created from classes 
	using virtual inheritance is 
	usually larger and slower in 
	performance than it would be with 
	non-virtual inheritance. 
	(see Item 40 of Effective C++).
*/

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
