// factory_function_interface.cpp

#include "../../my_essentials.hpp"

/*
 To simplify the procedure for coding the factory function at each time let us generalize this factory function.
*/
// First, include the header file of given virtual function
#include "4l_investor_virtual_function.h"
// Second, add aliases Base and Derived1, Derived2, etc. 
// to the given virtual function's base and dirived classes, respectively.
// For aliases, we can use a keyword 'typedef'

// EDIT ALIASES 
typedef advisor Base;
typedef investor1 Derived1;
typedef investor2 Derived2;
//-------------------------------

/*
auto my_factory_function_alais1
	= [&]()mutable{&Base::make_portfolio;};

auto my_factory_function_alias2
	= bind(&Base::make_portfolio);

//Though these alias like objects may represent the base function 'make_portfolio', 
//they cannot perform runtime switching of virtual function
//so we need to use the exact name of the virtual function (here it is "make_portfolio") at approiate place below.
*/

class my_factory
{
public:
    virtual unique_ptr<Base>createBase() = 0;
};
 
//ADD THIS PRODUCTION CLASS FOR ADDITIONAL DIRIVED CLASSES:

class production1: public my_factory
{
public:
    unique_ptr<Base> createBase()
    {
        return unique_ptr<Base>(new Derived1());
    }
};
 
class production2: public my_factory {
public:
    unique_ptr<Base> createBase() {
        return unique_ptr<Base>(new Derived2());
    } 
};

//------------------------------------------------
 
class my_production {
public:
    my_production(unique_ptr<my_factory> factory)
    {
        unique_ptr<Base> my_dynamic
				(factory->createBase());
		my_dynamic->
		// PUT IDENTIFIER NAME OF THE VIRTUAL FUNCTION BELOW:
				make_portfolio();
		//---------------------------------------
	}
};
 
unique_ptr<my_factory> choose_Derived() {
    auto flag = '1';
    cout<<"\nEnter product number(1 or 2 or ...): ";
    cin >> flag;

	switch(flag){
	//ADD CASES FOR ADDITIONAL DIRIVED CLASSES:
	case '1':
		return unique_ptr<my_factory> 
				(new production1());
	//break; this break is not required because it has 'return' above, which finishes this function.
	case '2':
		return unique_ptr<my_factory> 
				(new production2());
	default:
		cout 
		<< "\nSorry, unknown product number: "
		<< flag<< endl;
		cout << "\nSo providing product1:\n";
		return unique_ptr<my_factory> 
				(new production1());
	}
}
 
 
int main()
{
	unique_ptr<int> x(new int);
    	unique_ptr<my_production> 
		my_new_production
		(new my_production(choose_Derived()));
	just_pause();
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
