// complexity_guarantees.cpp

#include "my_essentials.hpp"
#include <list>
#include <set>

void complexity_guarantees_demo(list<int>lis)
{
	generate(lis.begin(), lis.end(), 
		     []()->int{return rand()%10;});
	
	cout	<< "\n Let us define a list,"
		<< "\n lis	= (";
	for_each(lis.begin(), lis.end(), 
		[](int v){cout << v << ' ' ;});
	cout 	<< ");\n";
	
	cout	<< "\n (1) Inserting an element"
		<< " at any point in <list> has\n"
		<< " Complexity of "
		<< "\"Constant-Time\":\n"
		<< "\n lis.insert(an iterator,"
		<< " value say 88); \n Thus,";

	auto it = lis.begin();
	if(lis.size() > 2) {
		it++; it++;
		// Note: it += 2; is not allowed,
		// because, list does not have 
		// the random access iterator, 
		// so it cannot jump!
	}

	lis.insert(it,88);

	cout 	<< "\n lis	= (";
	for_each(lis.begin(), lis.end(), 
		[](int v){cout << v << ' ' ;});
	cout 	<< ");"
		<< "\n The insertion of value 88"
		<<" might have taken\n"
		<< "constant time  => " 
		<< 1
		<< " microseconds\n";
	

	cout 	<< "\n Let us construct a multiset" 
		<<" 'mse' from the list 'lis'\n";
	multiset<int> mse(lis.begin(), lis.end()); 
	cout 	<< " mse	= (";
	for_each(mse.begin(), mse.end(), 
		[](int v){cout << v << ' ' ;});
	cout 	<< ");\n"
		<< " Note: By default, the multiset\n"
		<< " has stored the elements in "
		<< " ascending order!\n";
	
	cout	<< "\n (2) Counting a value"
		<< " in a multiset has\n"
		<< " Complexity of "
		<< "\"Logarithmic-Time\":\n"
		<< "\n Number of 4 in mse = "
		<< " mse.count(4) = " 
		<< mse.count(4)
		<< "\n It might have taken\n"
		<< " logarithmic time log("
		<<mse.size()
		<<") => "
		<<log(static_cast<double>(mse.size()))
		<< " microseconds\n";
	cout	<< "\n (3) The associative containers" 
		<< " are defined\n with the member"
		<< " function count(), but the\n"
		<< " sequence containers are not" 			<< " defined with that\n"
		<< " function, so they need to use\n"
		<< " the STL algorithm count(), but" 
		<< " that has complexity of" 
		<<" \"Linear-Time\":\n";
	
	cout	<< "\n Number of 4 in lis = "
		<< "count(lis.begin(), lis.end(), 4)"
		<< " = "
		<< count(lis.begin(), lis.end(), 4)
		<< "\n It might have taken \n" 
		<< " linear time ("
		<< lis.size()
		<< ") => "
		<< lis.size()
		<< " microseconds\n";
}

int main()
{
	int sz = 10;
	list<int>lis(sz);
	complexity_guarantees_demo(lis);
	just_pause();
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
