// tuple_demo.cpp

#include "my_essentials.hpp"
#include <tuple>

typedef tuple<string, double, double> my_tuple;

ostream& operator<<(ostream& os, const my_tuple& mt)
{// get<argument number>(tuple):
	os	<< get<0>(mt) << '\t'
		<< get<1>(mt) << '\t'
		<< get<2>(mt) << endl;
	return os;
}

void demo_tuple()
{
	// constructor: 
	my_tuple stock1("silver", 0.03, 0.05); 
	string ar1("some");
	double ar2(0), ar3(0);

	cout 	<< "\n(1) tie and ignore:\n";
	tie(ignore, ar2, ar3) = stock1;
	cout	<< " stock1: "
		<< ar1 << '\t' 
		<< ar2 << '\t' 
		<< ar3 << endl;

	cout	<< "\n(2) tuple_size<my_tuple>::value"
		<< " is " 
		<< tuple_size<my_tuple>::value
		<< endl;

	my_tuple stock2;
	cout	<<"\n(3) make_tuple:\n";
	stock2 = make_tuple("gold", 0.02, 0.08);

	cout	<< "\n(4) swap and cout using get:\n";
	stock1.swap(stock2);
	cout	<< " stock1: " << stock1 
			<< " stock2: " << stock2
			<< endl;
	
	cout	<< "(5) There are three different "
		<< "ways of deducing the type of "
		<< "a given tuple element\n"; 
	
	// (a) using tuple_element:
	tuple_element<0, my_tuple>::type
		val_by_tuple_element 
			= std::get<0>(stock1); 

	// (b) using decltype
	decltype(get<0>(stock1)) 
		val_by_decltype = get<0>(stock1);
	// where, the decltype(x) returns 
	// the type of its parameter x

	// (c) using auto
	auto 
		val_by_auto = get<0>(stock1);

	cout	<< " (a) val_by_tuple_element = "
		<< val_by_tuple_element << endl
		<< " (b) val_by_decltype = "
		<< val_by_decltype << endl
		<< " (c) val_by_auto = "
		<< val_by_auto << endl;

}

int main()
{
	demo_tuple();
	just_pause();
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
