// generate_distribution.cpp

# include "my_essentials.hpp"

typedef function<double()> my_funct_T;

class my_random_generator
{

 mt19937 engine;
 
 void check_mt19937();

 // The Gaussian distribution:
 // It generates a perfect Gaussian distribution.

 double Gaussian(double mean, double std_dev){
 	static normal_distribution<double> 
			normal(mean, std_dev);
 	return normal(engine); 
 }
 //-----------------------------------------------

 // The uniform real distribution:
 // It generates a uniform real distribution.

 double uniform_real(double start, double stop){
 	static uniform_real_distribution<double> 
				circle(start, stop);  
 	return circle(engine); 
 }
 //-----------------------------------------------

 // The uniform integer distribution:
 // It generates a uniform integer distribution.

 double uniform_int(double start, double stop){
 	int x = static_cast<int>(start);
 	int y = static_cast<int>(stop);
 	static uniform_int_distribution<int> 
				circle(x, y);  
 	return circle(engine); 
 }
 //-----------------------------------------------
  
 void print_pdf_cdf(my_funct_T my_funct, 
				double plot_size);

 double start, stop;

public:

 my_random_generator() { check_mt19937();}

 void plot_random_numbers(int choice);

};
//==================================================

int main()
{
	cout	<< "\n Welcome to plotting random " 
	 	<< "distributions!\n";			
	my_random_generator rg;
	int choice;
	cout << "\n Choose:\n"
	<< "\t 1 for the Gaussian distribution;\n"
	<< "\t 2 for the Uniform real distribution;"
	<< "\n\t 3 for integer numbers like throwing"
	<< " a dice;\n\t\t press (1/2/3): ";
	cin >> choice;
	if(choice < 1 || choice > 3) {
	cout	<< "\n Sorry, error, unknown option: "
	 	<< choice
	 	<< "\n therefore, terminating!\n";
	 	just_pause();
	 	return 1;
	}

	rg.plot_random_numbers(choice);
}
//============================================

void my_random_generator::plot_random_numbers
					(int choice)
{	
	double plot_size(14);

	switch(choice){
case 1:
	{
	cout << "\n For the Gaussian distribution,\n"
	<< " please provide: mean" 
	<< " (for default enter zero): ";
	cin	>> start;
	cout	<< "\n\t standard deviation" 
	 	<< " (for default enter 1): ";
	cin	>> stop;
	
	plot_size += 3*abs(start); 
			// the std function abs() 
			// extracts the absolute value, 
			// for example abs(-7) is 7

	auto my_funct 
	   = [this](){return Gaussian(start, stop);};

	print_pdf_cdf(my_funct, plot_size);

	break;
	}

case 2:
	{
	cout	<< "\n For the Uniform real"
	 	<< " distribution,\n please provide:" 
	 	<< " start value" 
	 	<< " (for default enter -1): ";
	cin	>> start;
	cout	<< "\n\t\t stop value"
	 	<< " (for default enter 1): ";
	cin	>> stop;
	
	plot_size 
	+= 2*max( (start < -6)?(0-start-6):0 , 
				(stop > 6)?(stop-6):0); 
	
	auto my_funct = [this](){
		return uniform_real(start, stop);};

	print_pdf_cdf(my_funct, plot_size);

	break;
	}

case 3:
	{
	cout	
	 	<< "\n For the "	
	 	<< "Uniform integer distribution,\n"
	 	<< " please provide: start value " 
	 	<< "(for default enter 1): ";
	cin	>> start;
	cout	<< "\n\t\t stop value " 
	 	<< "(for default enter 6): ";
	cin	>> stop;
	
	plot_size += 2*max( 
		(start < -6)?	(0-start-6):0 , 
		(stop > 6)?(stop-6):0); 

	auto my_funct = [this](){
		return uniform_int(start, stop);};

	print_pdf_cdf(my_funct, plot_size);

	break;
	}

default:
	cout	<< "\n Sorry, an unknown case!\n";
	break;

	}
}
//===============================================

void my_random_generator::check_mt19937()
{
	mt19937 engine; 
	static long long random_number = 0;
	for(int i =0; i != 9999; ++i){
		engine();
	}
	random_number = engine();
	// If the implementation is correct then 
	// the 10000th consecutive 
	// invocation of a default-constructed 
	// object of type mt19937
	// shall produce the value 4123659995, 
	// ref. C++11 ISO statement.
	if(random_number == 4123659995){
	cout	<< "\n  Note:\n\t "
	<< "The pseudorandom number generator\n\t "
	<< "Mersenne twister: MT19937 has been" 
	<< " tested\n\t and it shows it is"
	<< "implemented properly.\n"; 
	}
	else {
		cout	<< "\nWarning:\n\t "
	<< "The pseudorandom number generator\n\t "
	<< "Mersenne twister: MT19937 has been " 
	<< "tested\n\t and it shows it is NOT "
	<< "implemented properly.\n"; 
	}
}
//==================================================

void my_random_generator::print_pdf_cdf
	(my_funct_T my_funct, double plot_size)
{
	int channels 
		= static_cast<int>(plot_size)*100;
	int iterations 
		= static_cast<int>(plot_size)*2500000;
	cout	<< "\n\t Your plot size will be of " 
	 	<< plot_size << " units"
	 	<< "\n\t and " << channels 
	 	<< " channels\n";

	cout	<< "\n\t Progressing with "
	 	<< iterations 
	 	<< " iterations,\n\t please wait!\n";

	vector<double> PDF(channels,0);
	// PDF means probability density function,
	// i.e, the probability at each point.

	double position = 0;

	auto smooth = [this](){
		return uniform_real(0.0,1.0);};

	for(int i = 0; i != iterations; ++i){
		position = my_funct()
		*channels/plot_size + channels/2.0;
		// zoom-in and move up for plotting.

		position = floor(position + smooth());
		// rounding with a random number for 
		// converting the continuous form of 
		// data to digital form.
		// The floor picks only the integer 
		// part and discards the decimal part.

		if( (position<channels) &&
				(position>=0) ){
			PDF[(int)position]++;
			// this type of sort is called
			// a histogram sort or 
			// a counting sort or 
			// a bucket sort or 
			// a pigeonhole sort. 
		}
	}
	
	vector<double> CDF(channels, 0);
	// CFD means cumulative distribution 
	// function,
	// i.e., the sum of PDFs from negative 
	// infinity to the present position. 

	double sum_so_far = 0;
	for(int i =0; i != channels; ++i){
		sum_so_far += PDF[i];
		CDF[i] = sum_so_far;
	}

	// Normalizing PDFs and CDFs, such that the 
	// total probability = 1.
	for(int i =0; i != channels; ++i){
		(PDF[i]) /= sum_so_far;
		CDF[i] /= sum_so_far;
	}
	
	cout << "\n\t Thanks for your patience, "
	     << "it is finished.\n";

	string outfile("my_distribution.xls"); // to 
	// write result for plotting
	sure_write(outfile);
	// auto foo = ofstream(outfile);
	ofstream foo;
	foo.open(outfile);
	foo << "Channel\tPDF\tCDF\n";
	for(int i =0; i != channels; ++i){
		foo 
		<< static_cast<double> // zoom-out
		 (i - channels/2.0)*plot_size/channels
		<< "\t" << PDF[i] << "\t" 
		<< CDF[i] << endl;
	}
		
	
//-----------------------------------------------

	// system_notepad:
	//
	// We will use an execute system command of 
	// syntax:
	// int system (const char * command);
	//
	// This is for opening  a given file 
	// in notepad in Microsoft Windows operating 
	// system:

auto system_notepad = [](string outfile){

	// if the operating system is NOT Windows
	// then just cout the name of the outputfile:
	#ifndef _MSC_VER

	cout<< "\n\tThe output file: "
		<< outfile
		<< " is ready!\n\n";

	// else use notepad:
	#else

	string xl = "notepad " + outfile;

	char *cmd=new char[xl.size()+1];
	unsigned short sz = xl.size();
	for(auto i = 0; i < sz; ++i) {
		cmd[i] = xl[i];
	}
	cmd[sz] = 0; 
	// because, C-style character 
	// strings are null terminated.
	system(cmd);
	delete[] cmd;

	#endif
};
// As this lambda function can be useful 
// in our future programs, let us add it 
// in our header file for the last chapter: 
// "my_essentials_and_concurrency.hpp"
//------------------------------------------
	
	system_notepad(outfile);

}
//==================================================
