// my_first_multithreaded_code.cpp

#include "my_essentials_and_concurrency.hpp"
#include <thread>
#include <chrono> // to measure the time duration
#include <cmath>

typedef unsigned int usi;

void my_funct(usi start, usi stop, usi iteration, const vector<double>& data, double& result)
{
	auto sum(0.0);
	unsigned int i, x;
	if(stop <= data.size() && start 
				< data.size())
	{	
	for(i = 0; i < iteration; ++i) // can be as big as million iterations
	for(x = start; x < stop; ++x) // for a given range of vector
				sum += sqrt(data[x]);
	}
	else
	{
		cout << "out of range (" 
		<< start << ", " 
		<< stop << ")\n";
		result = 0;
		return;
	}
	result = sum;
}

void initialize_data(vector<double>& data)
{
	auto incr = [](){
		static auto i = 1.0; return ++i;};
	generate(data.begin(), data.end(), incr);
}

int main()
{
	auto iterations = 1000000;
	auto n = 10000;
	vector<double>data(n);
	initialize_data(ref(data));
	
	cout 
	<< "\n Beginning of non-concurrent task:\n";

	auto non_concurrent_start = time_now();

	double sum;
	auto length = data.size();
	auto start = 0;
	auto stop = length;


	my_funct(start, stop, iterations, 
		ref(data), ref(sum)); 

	cout << "\n\t Sum of f(data) " 	
	     << "(by non-concurrent task)\n\t\t= " 
	     << sum  << endl;
	
	cout << "\n End of non-concurrent task; and "
	     << "\n Beginning of concurrent task:\n";

	auto non_concurrent_stop 
			= time_now();

	// Let us implement four threads and run the 
	// same task in concurrency. We can either 
	// split the iterations into four parts or 
	// distribute the length of data into four 
	// parts. Then get the results in separate 
	// variables, such as: 

	double sum1(0), sum2(0), 
		sum3(0), sum4(0);

	// Let us distribute the data into four parts:
	
	start = 0; stop = length/4;
	thread my_thread1 (my_funct, start, stop, 
		iterations, ref(data), ref(sum1)); 
/*	Please note that this function call is different 
	from normal function call, here the function 
	name my_funct is placed as the first parameter 
	for the thread my_thread1 and this is followed 
	by argument parameters of the my_funct. */  

	start = length/4; stop = length/2;
	thread my_thread2 (my_funct, start, stop, 
		iterations, ref(data), ref(sum2));

	start = length/2; stop = (length*3)/4;
	thread my_thread3 (my_funct, start, stop, 
		iterations, ref(data), ref(sum3));

	start = (length*3)/4; stop = length;
	thread my_thread4 (my_funct, start, stop, 
		iterations, ref(data), ref(sum4));

	my_thread1.join();
	my_thread2.join();
	my_thread3.join();
	my_thread4.join();

	cout << "\n\t Sum of f(data)  " 
	     << "(by concurrent task)\n\t\t= " 
	     << (sum1 + sum2 + sum3 + sum4) 
	     << endl;

	cout << "\n end of concurrent task;\n";

	
	auto concurrent_stop 
			= time_now();

	auto non_concurrent_duration
	= duration(non_concurrent_start,
			non_concurrent_stop);
	
	auto concurrent_duration
	= duration(non_concurrent_stop, 
				concurrent_stop);

	auto speed_ratio = non_concurrent_duration
				/concurrent_duration;
	cout << "\n Time duration taken by\n"
	<< "\t(a) non-concurrent task = "
	<< setprecision (3) // sets decimal
			// precision to 3 digits.
	<< non_concurrent_duration/1e9 
	<< " seconds; "
	<< "\n\t(b) concurrent task = "
	<< concurrent_duration/1e9
	<< " seconds\n"
	<< "\n Therefore, this concurrent task is "
      	<< speed_ratio
	<< " times faster "
	<< "than the non-concurrent task\n";
	just_pause();
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
