// exception_unsafe_optimized_number_of_threads.cpp

#include "my_essentials_and_concurrency.hpp"
#include <thread>
#include <chrono> // high_resolution_time_library
#include <iomanip> // for manipulation such as
			// setw() and setprecision()

typedef unsigned int usi;

class optimum_threads{
	void find_numbers();
	void initialize_data();
	vector<thread> my_threads;
	vector<double> my_threads_duration; 
	vector<double>data;	
	usi iterations; 
	usi n;
	usi min_threads; 
	usi max_threads;

public:
	optimum_threads
		(int it, int sz, int min, int max):
		iterations(it), n(sz), 
		min_threads(min), max_threads(max) {
		data.resize(n); 
		find_numbers();
	}

	~optimum_threads() { // it recovers back 
	// if there is any thread resources 
	// left unrecovered 

		for(auto &th : my_threads)
			if(th.joinable()) 		
				th.join();
	}
		
};

int main()
{
	usi iterations = 500000;
	usi size_of_data = 10000;
	usi min_thread = 0;
	usi max_thread = 23;

	optimum_threads 
		ot(iterations, size_of_data, 
			min_thread, max_thread);

	just_pause();
}

void optimum_threads::initialize_data()
{
	auto incr = [](){
		static auto i = 1.0; return ++i;};
	generate(data.begin(), data.end(), incr);
}
					
void optimum_threads::find_numbers()
{	
	initialize_data();

	my_threads_duration.reserve(max_threads);

	auto my_funct = [this](usi iteration, 
					double& result){
	  result = 0; // assigning zero to result 
			// flushes out its old value and 
			// makes it ready for fresh
			// summation
	  for(unsigned int i = 0; i < iteration; ++i)
		for(auto x : data)
			result += sqrt(x);
	};


	// Let us implement by varying number of 
	// threads and run the task in 
	// concurrency. 

	for(auto threads = min_threads; 
		threads < max_threads; ++ threads)
	{
		cout 	<< "\n Beginning of "
			<< threads 
			<< "-thread concurrency:\n";
		
		auto start_time = time_now(); // 
		// time_now() returns present time. 
		// It along with its companion 
		// function duration(
		// 		time_point_initial,
		//		time_point_final) 
		// were added to the 
		// "my_essentials_and_concurrency.hpp" 
		// at the section 
		// '(5.9) Time facilities'
		
		my_threads.clear(); // clears previous 
		// contents of the vector 'my_threads'
		my_threads.shrink_to_fit(); 
		// this command along with clear() 
		// makes an empty vector of size zero.
		my_threads.reserve(threads); 
		// make reservation for new size
	   
             vector<double>sum_in_thread(threads,0);

		static double sum0 (0.0);
		
		if(threads == 0)
		{
		cout << "\n Here, zero thread" 
		<< " means non-concurrent task:\n";
		// so, we are not using threaded 
		// function, but normal function:	
		my_funct(iterations, ref(sum0)); 
		cout << "\t Sum of f(data) " 	
		<< "(by non-concurrent task)\n\t\t= " 
		<< sum0  << endl;
		auto end_time = time_now();

		my_threads_duration.push_back
		(duration(start_time, end_time));

		cout << " End of "
		<< threads << "-thread concurrency:\n"
		<< "-------------\n";
		continue; // this command 'continue'
			// makes the code to skip the 
			// following and go to the next 
			// level in the for loop 
		}
		
		// Let us divide the iterations 
		// among threads:
		auto it = iterations/threads; 
		// allocated iterations per thread
		auto mod = iterations%threads; 
		// remaining iterations will be 
		// added to the first thread
		

		for(usi i = 0; i < threads; ++i)
		{
			my_threads.push_back	(thread(
			my_funct, (it+mod), 
			ref(sum_in_thread[i]))); 
			mod = 0;
		}

		for(auto &x : my_threads)
			x.join();

		auto total = 0.0;
		for(auto sum : sum_in_thread)
			total += sum;
		cout << " Sum = " << total << endl;
		
		// The floating-point types double, 
		// float are approximations of the 
		// mathematical real numbers, 
		// therefore using them may cause a 
		// very low percentage of error and 
		// that error is called rounding 
		// error, for example:
		auto percentage_of_rounding_error 
			= ((sum0 - total)/sum0)*100.0;
		if(percentage_of_rounding_error < 1.0) 
			percentage_of_rounding_error 
					*= -1.0;
		cout 	<< " Percentage of rounding"
			<< " error of double = " 
			<< percentage_of_rounding_error 
			<< " %" << endl;

		auto end_time = time_now();
		my_threads_duration.push_back
		(duration(start_time, end_time));

		cout << " End of "
		<< threads << "-thread concurrency:\n"
		<< "-------------\n";
	}
				
	array<char,300> sno = {  	
		'a', 'b', 'c', 'd', 'e', 'f', 
		'g', 'h', 'i', 'j', 'k', 'l',			
		'm', 'n', 'o', 'p', 'q', 'r',			
		's', 't', 'u', 'v', 'w', 'x', 
		'y', 'z'}; 

	string outfile("optimum_thread.xls"); // to 
	// write result for plotting 
	sure_write(outfile);
	auto fo = ofstream("optimum_thread.xls");

	cout << "\nResult:\n"
	<< "\nDuration in seconds for "
	<< "task carried out by\n"; 

	auto i = min_threads;
	auto speed = my_threads_duration.at(0);
	for(auto &t : my_threads_duration)
	{
		speed = my_threads_duration[0]/t; 
		cout<< "\n("<< sno[i]<< ") " 
			<< setw(2)<< i 
			<< ( (i >= 2)? " threads took"
					:" thread took ") 
			<< setw(6) << t/1e9 
			<< " seconds; i.e., " 
			<< setprecision(3)
			<< speed
			<< ( (speed >= 2)? " times "
						:" time ") 
			<< "faster!\n"; 
		fo  << i << '\t' << speed << endl;
		++i;
	}
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
