//expection_safe_user_friendly_finding_optimized_number_of_threads.cpp

#include "my_essentials_and_concurrency.hpp"
 
typedef unsigned int usi;

class optimum_threads{	

	// Sol1: Exception Safety: part 1 of 3:

	vector<future<double>> my_threads; // use of 
	// future with async can transfer exception
	vector<double> my_threads_duration; 
	vector<double>data;	
	void initialize_data();
	void find_numbers();
	usi iterations; 
	usi n;
	usi min_threads; 
	usi max_threads;

	// Sol2: User friendly: part 1 of 4:

	// Now the key function such as sqrt(),  
	// log() can be supplied by the user and this 
	// can be assigned as the 'user_function':

	typedef function<double(double)> T_udf; 
	T_udf user_function; 

public:
	optimum_threads
		(int it, int sz, int min, int max, 
						T_udf UDF):
		iterations(it), n(sz), 
		min_threads(min), max_threads(max),
		user_function(UDF) {
		find_numbers();
	}	
};

int main()
try
{
	usi iterations = 500000;
	usi size_of_data = 10000;
	usi min_thread = 0;
	usi max_thread = 23;

	// Sol2: User friendly: part 2 of 4:

	auto UDF // User-Defined Function 
		= [](double x){ 
			
	//Sol1: Exception safety: part 2 of 3:

		if(x < 0) 
			throw ( "input of sqrt() cannot have a negative value!");			
		return sqrt(x);
	};

	// Sol2: User friendly: part 3 of 4:

	// UDF is supplied as a parameter to 
	// the constructor argument for using it
	// as the key function of the class

	optimum_threads 
		ot(iterations, size_of_data, 
			min_thread, max_thread, UDF);
	
	just_pause();
}

catch (exception& e) {
    cerr << "\n Standard Library exception caught: " 
	  << e.what() << endl;
    just_pause();
}

// Sol1: Exception safety: part 3 of 3:

catch (char* e) {
    cerr << "\n Exception caught: " << e << endl;
    just_pause();
}

catch(...){
	cout	<< "\n Problem: there is an unknown " 
		<< "exception!\n";
	just_pause();
	return 2;
}

void optimum_threads::initialize_data()
{
	data.resize(n);
	auto incr = [](){
		static auto i = 1.0; return ++i;};
	generate(data.begin(), data.end(), incr);
}
					
void optimum_threads::find_numbers()
{	
	initialize_data();

	my_threads_duration.reserve(max_threads);

	auto my_funct = [this](usi iteration) {
		double result = 0; 
		for(unsigned int i = 0; i < iteration; 
							++i)
		for(auto x : data) 

		// Sol2: User friendly: part 4 of 4:

			result += user_function(x);
		return result;
	};

	// Let us implement in varying number of 
	// threads and run the task in 
	// concurrency. 

	for(auto threads = min_threads; 
		threads < max_threads; ++ threads)
	{
		cout 	<< "\n Beginning of "
			<< threads 
			<< "-thread concurrency:\n";
		
		auto start_time = time_now();  

		my_threads.clear(); 
		my_threads.shrink_to_fit(); 
		my_threads.reserve(threads); 

		static double sum0 (0.0);

		if(threads == 0)
		{
		cout << "\n Here, zero thread" 
		<< " means non-concurrent task:\n";
		sum0 = my_funct(iterations); 
		cout << "\t Sum of f(data) " 	
		<< "(by non-concurrent task)\n\t\t= " 
		<< sum0  << endl;
		auto end_time = time_now();

		my_threads_duration.push_back
		(duration(start_time, end_time));

		cout << " End of "
		<< threads << "-thread concurrency:\n"
		<< "-------------\n";
		continue; 
		}
		
		// Let us divide the iterations 
		// among threads:
		auto it = iterations/threads; 
		// allocated iterations per thread
		auto mod = iterations%threads; 
		// remaining iterations will be 
		// added to the first thread
		

		for(usi i = 0; i < threads; ++i)
		{
			my_threads.push_back ( 
			async(my_funct,(it+mod))); 
			mod = 0;
		}

		auto total = 0.0;
		for(auto &x : my_threads)
			total += x.get();

		
		auto percentage_of_rounding_error 
			= ((sum0 - total)/sum0)*100.0;
		if(percentage_of_rounding_error < 1.0) 
			percentage_of_rounding_error 
					*= -1.0;
		cout 	<< " Percentage of rounding"
			<< " error of double = " 
			<< percentage_of_rounding_error 
			<< " %" << endl;

		auto end_time = time_now();
		my_threads_duration.push_back
		(duration(start_time, end_time));

		cout << " End of "
		<< threads << "-thread concurrency:\n"
		<< "-------------\n";
	}
				
	array<char,300> sno = {  	
		'a', 'b', 'c', 'd', 'e', 'f', 
		'g', 'h', 'i', 'j', 'k', 'l',			
		'm', 'n', 'o', 'p', 'q', 'r',			
		's', 't', 'u', 'v', 'w', 'x', 
		'y', 'z'};

	string outfile("optimum_thread.xls");
	sure_write(outfile);
	auto fo = ofstream("optimum_thread.xls");

	cout << "\nResult:\n"
	<< "\nDuration in seconds for the "
	<< "task carried out by\n"; 

	auto i = min_threads;
	auto speed = my_threads_duration.at(0);

	for(auto &t : my_threads_duration)
	{
		speed = my_threads_duration[0]/t; 
		cout 	<< "\n("<< sno[i]<< ") " 
			<< setw(2)<< i 
			<< " threads took " 
			<< setw(6) << t/1e9 
			<< " seconds; i.e., " 
			<< setprecision(4)
			<<  speed
			<< ( (speed >= 2)? " times "
						:" time ") 
			<< "faster!\n";
		fo << i << '\t' << speed << endl;
		++i;
	}
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
