// Non_Concurrent_house_pricing.cpp

// Warning: This code has little to do with 
// any real life financial calculations, 
// but its focus is C++ only.

#include "my_essentials_and_concurrency.hpp"

typedef const vector<double> cvec;
typedef const double cdouble;
typedef unsigned int usi;
typedef const usi cusi;
typedef const string cstring;

class house{
	
	cdouble S0;
	cusi T;
	cdouble dt;
	cdouble sqrt_dt;
	cdouble r;
	cdouble r_dt;
	cdouble sigma;
	cstring excel_out; 


	vector<double>	
		very_big_data, 
		very_big_data_mini, 
		very_big_data_maxi;
	
	void simulate_a_unit(vector<double>&);
	void simulate_as_many_as(usi iterations);
	void write_for_excel(cvec&, cvec&, cvec&);

public:

	house(double, double, double, usi, string);
	void price_generate(usi total_iterations);
};
//--------------------------------

int main()
{	
	usi iterations = 6000000;
	double market_price = 3000;  
	double price_gain = 2.1/100; //=> so it is a  
			// bull (growing) market	
	double price_fluctuation = 10.0/100; 
	usi period = 252*24; // a year has 252 
	// working days, which is converted into 		
	// hours to get hourly chart
	string excel_outfile("house_price_chart.xls"); 
	sure_write(excel_outfile);

	house hs(market_price, 
		price_gain, price_fluctuation, 
		period, excel_outfile);

	auto start = time_now();

	cout << "\n\t Welcome to the Non-Concurrent"
	     << "\n\t Monte Carlo simulation with"
	     << "\n\t " << iterations
            << " iterations.";

    hs.price_generate(iterations);		
	
	auto stop = time_now();
	cout 	<< "\n\t" << iterations 
		<< " of iterations took " 
		<< duration(start,stop)/1e9 
		<< " seconds \n";

	// system("notepad house_price_chart.xls"); 
	system_notepad(excel_outfile);
}
//--------------------------------

house::house(double market_price, double growth_rate, 
	double price_fluctuation, usi period, 
	string excel_outfile) 
	
	: 

	// (a) initializations of 
	//     S0, r, sigma, T and excel_out:
	S0(market_price), r(growth_rate), 
	sigma(price_fluctuation), 
	T(period), excel_out(excel_outfile),

	// (b) assignments for 
	//	   dt, sqrt_dt, r_dat and vectors:
	dt(1.0/T), sqrt_dt(sqrt(dt)), r_dt(r*dt), 
	very_big_data(T, 0.0), 
	very_big_data_mini(T, 0.0), 
	very_big_data_maxi(T, 0.0) {}
//--------------------------------

void house::simulate_a_unit(vector<double>& data){

	usi i;
	double S;
	double dX(0.0);
	double dS(0.0);

	S = data[0] = S0;	

	for(i = 1; i < T; ++i) {
		dX = phi()*sqrt_dt;
		dS = S*(r_dt + sigma*dX);
		S += dS;
		data[i] = S;
	}
}
//--------------------------------

void house::simulate_as_many_as(usi it) {

	cout << "\n\n\t   Work is in progress!\n";

	usi i(0), j(0);
	vector<double> data(T,0.0);
	vector<double> big_data(T, 0.0);
	vector<double> big_data_mini(T, 100*S0);
	vector<double> big_data_maxi(T, 0.0);	

	for(i = 0; i < it; ++i){
		
		simulate_a_unit(ref(data));
		
		for(j = 0; j < T; ++j)
			big_data[j] += data[j];

		if(big_data_mini[T-1] > data[T-1])
			big_data_mini = data;
		else
			if(big_data_maxi[T-1] 
					< data[T-1])
			big_data_maxi = data;
	}

	for(auto& x: big_data) x /= it;

	write_for_excel(	ref(big_data), 
				ref(big_data_mini), 
				ref(big_data_maxi));
};
//--------------------------------

void house::price_generate(usi iterations) {

	simulate_as_many_as(iterations);

	auto fexcel = ofstream(excel_out);
	for(usi i = 0; i< T; ++i) 
		fexcel << i*T*dt << "\t" 
			<< very_big_data_mini[i] << "\t" 
			<< very_big_data[i] << "\t" 
			<< very_big_data_maxi[i] 
			<< endl;
	fexcel.close();	
}
//--------------------------------
	
void house::write_for_excel(
		cvec& big_data,
		cvec& big_data_mini,
		cvec& big_data_maxi){
	static mutex my_mutex;
	for(usi k = 0; k < T; ++k)
	{ 
		lock_guard<mutex> guard(my_mutex);

		very_big_data[k] += big_data[k];

		very_big_data_mini[k] 
				+= big_data_mini[k];

		very_big_data_maxi[k] 
				+= big_data_maxi[k];
	}
}
//--------------------------------

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
