// common_operators.cpp

#include "ans11_common_constructors_and_operators.h"

void my_container_demo
	::operators_for_all_containers()
{
	cout << "\nList of operators that are "
		 << " common to all containers:\n";
	//let us generate a sample vector
	investment_data inv_data;
	array<investment,3> my_array = { 
				inv_data.bank1, 
				inv_data.bond1, 
				inv_data.real_estate1
	};	
	vector<investment> my_vec(my_array.begin(),
				my_array.end() );
	cout_container(
	"\nLet us define a vector:\n\tmy_vec",
	my_vec);

	// begin(), end():
	cout<< "\nIterators:\n"
		<< "(1) begin() and (2) end():\n";
	if(!my_vec.empty()){
	cout<< "\t*my_vec.begin() = "
		<< *my_vec.begin() 
		<< ";\n\t*(my_vec.end()-1) = "
		<< *(my_vec.end()-1) << endl; 
		/* yes -1 is there because the end()
			represents one beyond the 
			last element.*/
	cout<<"...........................\n";

	// rbegin(), rend():
	cout<< "\nReverse Iterators:\n"
		<< "(3) rbegin() and (4) rend():\n"
		<< "\t*my_vec.rbegin() = "
		<< *my_vec.rbegin() 
		<< ";\n\t*(my_vec.rend()-1) = "
		<< *(my_vec.rend() - 1) << "\n\n";
	cout<<"...........................\n";
	
	// cbegin(), cend(), crbegin(), crend():
	const vector<investment>
				my_const_vec = my_vec; 
	vector<investment>::const_iterator it;
	cout<< "\nConstant Iterators, "
		<< "Constant Reverse Iterators:\n"
		<< "\nElements of a constant "
		<< "container such as\n\t"
		<< "\"const vector<investment> "
		<< "my_const_vec = my_vec;\" \n"
		<< "cannot be accessed by an iterator"
		<< " unless it is a const_iterator,"
		<< "i.e.,\n\t\"vector<investment> "
		<< "::const_iterator it;\".\n"
		<< "To support const_iterator C++"
		<< " defines four operators:\n\t"
		<< "cbegin(), cend(), crbegin() and "
		<< "crend().\n";
	cout<<"...........................\n";
	}	

	if(my_vec.size() >= 3 )
	{
	// =, ==, !=, <, <=, >, >= :
	cout<< "\nRelational Operators:\n"
		<< "(5)=, (6)==, (7)!=, (8)<"
		<< "\n(9)<=, (10)>, and (11)>= :"
		<< endl;
	auto i = my_vec.begin();
	auto j = i+1;
	auto k = i+2;
	if(i == my_vec.begin())
	  if(i<j)
	   if(k > j)
	     if(my_vec.end() >= k) {
	cout<< "\nYes, it is a successful "
		<< "performance of relational "
		<< "operators =, ==, <, >, >=\n";
	cout<<"..........................\n\n"; 
		}
	}

	// emplace():
	vector<investment>my_new_vec= my_vec;
	cout
	<< "(12) emplace, "
	<< "which means element constructed in "
	<< "place. For example let us construct "
	<< "a vector my_new_vec as a copy of "
	<< "my_vec and apply two ways of "
	<< "emplace operation to grow the my_new_vec:\n"
	<< "\"my_new_vec.emplace_back" 
	<< "(inv_data.share1);\"\n"
	<< "\"my_new_vec.emplace(my_new_vec.begin(), "
	<< "inv_data.gold);\"";	
	my_new_vec.emplace_back(inv_data.share1);
	my_new_vec.emplace(
		my_new_vec.begin(), 
		inv_data.gold);
	cout << "\nafter emplace operation it becomes:";
	cout_container("\n\tmy_new_vec", my_new_vec);
	cout<<"...........................\n\n";

	// swap():
	cout<< "(13) swap between "
	    << "two vectors:\n"
	    << "\"my_vec.swap(my_new_vec);\"\n";
	my_vec.swap(my_new_vec);
	cout_container("\tThus:\n\tmy_vec", my_vec);
	cout_container("\n\tmy_new_vec", my_new_vec);
	cout<<"...........................\n\n";

	// size(), max_size(), empty():
	cout	<< "(14) size() - length of the "
    		<<  "container at present;\n"
    		<< "(15) max_size() - maximum "   
		<< " possible length of the "
		<< "container\n"
		<< "(16) empty() - true if the"
		<< " container"
		<< "is empty:";
	
	cout	<< "\n\tmy_vec.size() = "
		<< my_vec.size() 
    		<< ";\n\tmy_vec.max_size() = "
		<< my_vec.max_size()
    		<<";\n\tmy_vec.empty() = ";
	my_vec.empty()?
		cout<<"empty"
		:cout<<"not empty"<<endl;
	cout<<"...........................\n\n";

	// erase():
	cout<< "(17) erase() - erase a range of "
	    << "elements or a single element:"
	   
	    << "\n\tbefore erase:\n\t";
	cout_container("my_vec", my_vec);
	
	my_vec.erase(my_vec.begin(), my_vec.end()-1);
	cout<< "\n\tafter erasing elements from "
	    << "begin() up to end()-1:\n\t";
	cout_container("my_vec", my_vec);
	
	my_vec.erase(my_vec.begin());
	cout<< "\n\tafter erasing element at "
	    << "begin():\n\t";
	cout_container("my_vec", my_vec);
	cout<<"...........................\n\n";	

	// insert():
	cout<< "(18) insert() - inserts "
	    << "element(s):"
	    << "\n\tbefore insert():\n\t";
	cout_container("my_vec", my_vec);
	my_vec.insert(
		my_vec.begin(), 	
		my_new_vec.begin(), 
		my_new_vec.end() );
	cout<< "\n\tafter insert() of "
	    << "my_new_vec in my_vec:\n\t";
	cout_container("my_vec", my_vec);
	my_vec.insert(my_vec.end(), inv_data.gold);
	cout<< "\n\tafter insert() of an "
	    << "element \"inv_data.gold\":\n\t";
	cout_container("my_vec", my_vec);
	cout<<"...........................\n\n";

	// clear():
	cout<< "(19) clear() - erases "
	    << "all elements in a vector:"
	    << "\n\tbefore clear():\n\t";
	cout_container("my_vec", my_vec);
	my_vec.clear();
	cout << "\n\tafter clear():\n\t";
	cout_container("my_vec", my_vec);
	cout<<"..........................."<<endl;
}

//

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
