// additional_operations_of_list.cpp

#include "my_essentials.hpp"
class my_list_demo{
public:
	void operators_of_list_template();
};

int main()
{
	my_list_demo vdemo;
	vdemo.operators_of_list_template();
	just_pause();
}

void my_list_demo::operators_of_list_template()
{
	cout << "\nList of special \"operators\" of std::list<T>:\n";
	//let us generate a sample list:
	list<double>my_lis(6);
	generate(my_lis.begin(), my_lis.end(), 
		[]()->double{static double d=1; return d++;} );
		//[]()->double{return(rand()%10);});
	cout_container("\n  Let us define a list:\n  my_lis", my_lis);

	my_lis.push_front(7531);
	cout_container("\n  operator (1)push_front():"
		"\n\tmy_lis.push_front(7531) leads to\n\tmy_lis",my_lis);	
	
	my_lis.pop_front();
	cout_container("\n  operator (2)pop_front():"
		"\n\tmy_lis.pop_front() leads to\n\tmy_lis",my_lis);
	
	my_lis.remove(5);
	cout_container("\n  operator (3)remove():"
		"\n\tmy_lis.remove(5) leads to\n\tmy_lis",my_lis);
	
	my_lis.remove_if([](double d){return d >= 4;});
	cout_container("\n  operator (4)remove_if():"
		"\n\tmy_lis.remove_if([](double d){return d >= 4;}) leads to\n\tmy_lis",my_lis);

	cout<<"\n  operator(5) merge():";
	list<double> new_lis1(5), new_lis2(5);
	generate(new_lis1.begin(), new_lis1.end(), 
		[]()->double{static double d=1; return d++;} );
	generate(new_lis2.begin(), new_lis2.end(), 
		[]()->double{static double d=6; return d++;} );
	cout_container("\n\tLet us have a new list:\n\tnew_lis1",new_lis1);
	my_lis.merge(new_lis1);
	cout_container("\tLet us merge it with my_lis by my_lis.merge(new_lis1)\n\tmy_lis",my_lis);
	cout<<"\tNote: it merges in ascending order.";
	
	
	cout<<"\n\n  operator(6) splice():";
	cout_container("\n\tLet us have another new list:\n\tnew_lis2",new_lis2);
	cout << "\tmy_lis.splice(my_lis.begin(), new_lis2) is same as"
		 <<" my_lis.merge(new_lis2), but not in ascending order";
	cout<<"\n\tIt also helps in merging as user specified, for example\n";
	cout << "\t\"my_lis.splice( ++my_lis.begin(), new_lis2, new_lis2.begin(), -- --new_lis2.end() );\""
		 <<" says start merging from first element to last but one element of new_lis "
		 <<"at second position of my_lis, thus we have\n\t";
	my_lis.splice( ++my_lis.begin(), new_lis2, new_lis2.begin(), -- --new_lis2.end() );
	cout_container("my_lis",my_lis);
	
	cout<<"\n  operator(7) unique():";
	my_lis.unique();//default predicate is equal_to<T>()
	cout_container("\n\tIt removes the right hand side elment of "
		"dynamic pairs of adjustant elements which satisfy a given predicate;"
		" by default it is \"equal_to<T>()\", so my_lis.unique() leads to"
		"\n\tmy_lis", my_lis);

	my_lis.unique([](double x, double y){return x+1==y;});
	cout_container("To remove a specific pattern from the list, "
		"a lambda function can be used, for example"
		"\"my_lis.unique([](double x, double y){return x+1==y;})\", "
		"leads to \n\tmy_lis",my_lis);

	cout<<"\n  operator(8) sort():";
	my_lis.sort(greater<double>());
	cout_container("\n\tBy default it sorts in ascending order using "
		"the predicate less<T>(), but with the predicate greater<T>(), \""
		"my_lis.sort(greater<double>())\" "
		"sorts in descending order, i.e., \n\tmy_lis", my_lis);
	cout<<"\n\t Let us remove the repeated elements "
		<<"in the sorted list by using \"my_lis.unique()\", thus\n\t";
	my_lis.unique();
	cout_container("my_lis", my_lis);

	cout<<"\n  operator(9) reverse():";
	my_lis.reverse();
	cout_container("\n\tThe statement \"my_lis.reverse()\" reverses the list, thus, \n\tmy_lis", my_lis);
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
