// operators_of_map_and_unordered_multimap.cpp
#include "my_essentials.hpp"
#include <unordered_map>
#include <map>

void operators_of_unordered_multimap
(unordered_multimap<string, double>& ummp)
{		
	cout << "\nTesting the Operators of Unordered Associative Containers:\n";
	//auto ummp = ummap;
	cout_container2T
		("Let us consider an unordered_multimap\n\tummp",ummp);
	
	cout << "\n(1) size_type bucket(K):\n";
	//for(auto& ele: ummp) // though this range_based for loop is the best, to maintain compatibility with VS10 we adopt the following classical procedure:
	auto ele = ummp.begin();
	auto ele_end = ummp.end();
	for(; ele != ele_end; ++ele)
		cout	<< "The element (" 
			<< ele->first 
			<< ", " << ele->second 
			<< ") is in the bucket index " 
			<< ummp.bucket(ele->first) 
			<< endl;	
	cout <<"-------------------------\n";

	cout<< "\n(2) size_type bucket_size(bucket_index):";
	cout<< "\nNumber of elements in each bucket:\n";
	auto btcount = ummp.bucket_count();
	for(unsigned int i =0; i< btcount; ++i)
		cout << "\tBucket #" << i 
		<< " has " << ummp.bucket_size(i) 
		<< endl;	
	cout <<"-------------------------\n";


	cout<< "\n(3) size_type bucket_count():";
	cout<< "\nNumber of buckets in the container = " << ummp.bucket_count() << endl;	
	cout <<"-------------------------\n";

	cout<< "\n(4) float load_factor():\n";
	cout<< "The load factor of the container\n"
	<< " = size/count = " << ummp.size()
	<< "/" << ummp.bucket_count() << "\n = " 
	<< ummp.load_factor() << endl;	
	cout <<"-------------------------\n";

	cout<< "\n(5a) float max_load_factor():\n"
	<< "The present maximum load factor = "
	<< ummp.max_load_factor()
	<< ",\nand if the load factor reaches\n"
	<< "the max_load_factor then the\n"
	<< "max_load_factor will automatically alter.\n";	
	cout <<"-------------------------\n";
	
	cout<< "\n(5b) void max_load_factor\n"
	<< "	(float my_desired_load_factor):\n"
	<< "can assign desired load_factor, say 0.1\n";
	ummp.max_load_factor(0.10f);//where, 'f' indicates 
//that '0.10' should be considered as float.
	cout <<"-------------------------\n";
	
	cout << "\n(6) void rehash\n"
	<< "	(size_type desired_number_of_buckets):\n"
	<< "it rebuilds the container with\n"
<< "the desired_number_of_buckets, say '10'\n";
	ummp.rehash(100);	
	cout <<"-------------------------\n";

	cout 	<< "\nTherefore, the new value of\n"
		<< "the number of buckets = "
		<< ummp.bucket_count() << endl
		<< " the load factor = "
		<< ummp.load_factor() << endl
		<< " the maximum load factor = "
		<< ummp.max_load_factor() << endl;	
	cout <<"-------------------------\n";

	cout<< "\n(7) size_type max_bucket_count():\n";
	cout	<< "The maximum number of buckets\n"
		<< "that this container can have in\n"
		<< "this computer system\n"
		<< "and implementation = " 
		<< ummp.max_bucket_count() << endl;
	cout<<"-------------------------\n";	
	
	cout<< "\n(8) hasher hash_function()(key_type):\n"
	  << "it returns Hash as a unique value\n"
	  << "of type size_t based on its\n"
	  << "key_type argument\n";
	//unordered_multimap<string, double>::hasher hfn = ummp.hash_function(); 
// but simply,
	auto hfn = ummp.hash_function();
	cout<< "yes, : " << hfn("yes, ") << endl
	<< "it: "  << hfn("it")  << endl
<< "is.: "  << hfn("is.")  << endl;
	//cout << fn(10); // error: 10 is the given key_type: string	
	cout <<"-------------------------\n";
	
	cout<< "\n(9) Predicate key_eq()(K1, K2):\n"
	<< "it returns 'true' if K1 and K2 are equivalent.\n";
	//unordered_multimap<string, double>::key_equal kefn = ummp.key_eq(); 
// but simply,
	auto kefn = ummp.key_eq();
	auto 
	  m1 = "Mango",
	  m2 = "mango",
	  m3 = "mango";
	cout << m1 << " and " << m2 << " are ";
	kefn(m1,m2)? 
		cout << "equivalent": 
		cout << "not equivalent";
	cout	<< "\n\tand\n"
		<< m2 << " and " << m3 << " are ";
	kefn(m2,m3)? 
		cout<< "equivalent": 
		cout << "not equivalent";	
	cout <<"\n-------------------------\n";

	cout<< "\n\n(10) Allocator get_allocator():\n"
	<< "it returns the the allocator involved\n"
	<< "in constructing the container.\n";
	//unordered_multimap<string, double>::allocator_type get_alloc = ummp.get_allocator(); 
// but simply,
	auto get_alloc = ummp.get_allocator();
	allocator<std::pair<const string, double> > default_alloc;
	cout << "Here, get_allocator() provided the";
	(get_alloc == default_alloc)? 
	  cout << "correct" : cout << "incorrect";
	cout << " allocator\n";	
	cout <<"-------------------------\n";

	cout<< "\n(11) const_iterator find(K):\n";
	auto find_test = [ummp](string st) {
	auto fi = ummp.find(st);
	cout << "ummp.find(\""<<st<<"\") finds (";
	if(fi != ummp.end())	 
	  cout << fi->first << ", " 
		<< fi->second
		<< ")\n";
	else
	  cout << "Null)\n";
	};
	find_test("Gold");
	find_test("Aluminum");
	find_test("Crude_oil");	
	cout <<"-------------------------\n";

	cout<< "\n(12) size_type count(K) :\n";
	auto count_test = [ummp](string st) {
	 auto fi = ummp.count(st);
	 cout << "ummp.count(\""<<st<<"\") = ";	 
	 cout << fi << "\n";
	}; 
	count_test("Gold");
	count_test("Aluminum");
	count_test("Crude_oil");	
	cout <<"-------------------------\n";

	cout<< "\n(13) pair<it,it>equal_range(K) :\n";
	auto eqrange_test = [ummp](string st) {
	 auto er = ummp.equal_range(st);
 cout 	<< "\nummp.equal_range(\""<<st
		<<"\") =\n";
 if(er.first != er.second){
	   for(;er.first != er.second; ++er.first)
		   cout<< "\t(" << er.first->first 
			<< ", "<< er.first->second 
			<< ")\n";
	 }
	 else
	   cout << "\t(Null)\n";
	}; 
	eqrange_test("Gold");
	eqrange_test("Aluminum");
	eqrange_test("Crude_oil");	
	cout <<"-------------------------\n";

	cout << "\n(14) operator[] for map and unordered_map:\n";
	unordered_map<string, double> ump(ummp.begin(), ummp.end());
	cout<< "From the unordered_multimap:\n";
	cout_container2T( "ummp", ummp);
	cout 	<< "let us construct an "
		<< "unordered_map:\n";
	cout_container2T( "ump", ump);
	cout	
<< "\nUsing the operator[], we can insert"
	<< "\na key with the value and we can "
	<< "access the value for a given key,\n"
	<< "for example, let us insert:\n"
	<< "ump[\"Silver\"] = 0.22; "
<< "ump[\"Coal\"] = 0.09;\n";
	ump["Silver"] = 0.22;
	ump["Coal"] = 0.09;
	cout_container2T
		( "this increases the ump", ump);
	cout<<"\nLet us access the value corresponding to the key \"Gold\":\n"
	<< "ump[\"Gold\"] = "<< ump["Gold"] << endl;
	cout <<"-------------------------\n";
}
//////////////////////////////////////////

void operators_of_multimap(const multimap<string, double>& mmp)
{	
	cout<< "\nIllustration of the operators of" 
	<< " multimap:\n" 
	<< "The operators: find, count and" 
	<< " equal_range are the same as illustrated in" 
	<< " the above illustartion for the" 
	<< " unordered containers. Therefore, here" 
	<< " we will illustrate the remaining four"
	<< " operators: key_comp, value_comp," 
	<< " lower_bound and upper_bound\n";
	cout <<"-------------------------\n";

	cout << "\n(1) key_compare key_comp()(key_type, key_type):\n"
	<< "It returns a copy of the comparison function that is used for sorting the ordered associative container.";
	auto its_kcomp = mmp.key_comp();
	cout << "\n The given container sorts in ";
	auto test_comp = its_kcomp("A", "Z");
	if(test_comp)
		cout << "ascending order!\n";
	else
		cout << "decensending order!\n";
	//test_comp = its_kcomp(10,20); // error why?
	// because the key_type is string in this 
	// multimap<string, double> and not the 
	// multimap<int, double>
	cout <<"-------------------------\n";
	
	cout<< "\n(2) value_compare value_comp()(value_type, value_type):\n"//where, value_type means pair<K,V>
	<< "It returns a copy of the comparison function that is used for sorting the ordered associative container.\n"
	<< "\nIt is nearly the same as the key_comp(K,K), and only the types of their arguments differ.\n";
	auto its_vcomp = mmp.value_comp();
	pair<string, double> x ("A", 1.2);
	pair<string, double> y ("Z", 3.1);
	test_comp = its_vcomp(x,y);
	cout << "\n The given container sorts in ";
	if(test_comp)
		cout << "ascending order!\n";
	else
		cout << "decensending order!\n";
	cout <<"-------------------------\n";

	cout<< "\n(3) iterator lower_bound(K):\n"
	<< "It returns an iterator to the first element in the container whose key value is greater than or equal to(>=) the specified key value in the argument.\n"
	<< "\n(4) iterator upper_bound(K),\n"
	<< "It returns an iterator to the first element in the container whose key value is greater than(>) the specified key value in the argument.\n";
	cout_container2T("\nThe given multimap\n\tmmp", mmp);
	auto test_bound = [mmp](string key){
	 auto lb = mmp.lower_bound(key);
	 auto ub = mmp.upper_bound(key);
	
	 cout	<< "\nFor the key \""
		<< key << "\"\n"
		<< "\tthe lower_bound element is: (";
	 if(lb != mmp.end())
		cout	<< lb->first << ", " 
			<< lb->second << ")\n";
	 else
		cout	<< "not exist)\n";
	
	 cout	<< "\tthe upper_bound element is: (";
	 if(ub != mmp.end())
		cout	<< ub->first << ", " 
			<< ub->second << ")\n";
	 else
		cout	<< "not exist)\n";
	};
	test_bound("Gold");
	test_bound("Crude_oil");
	test_bound("S&P");
	test_bound("Diamond");
	cout << "\t This is so because the letter 'D' is after 'C' and before 'G'.\n"; 
	test_bound("Zebra");
	cout << "\t This is so because the letter 'Z' is greater than the first letter 'S' of the last element key 'S&P'.\n"; 
	cout <<"-------------------------\n";
}

int main()
{	// Let us define, 
	// a unordered_multimap
	typedef unordered_multimap<string, double> 
						my_map;
	typedef my_map::value_type my_value_type;
	my_map ummp;
	ummp.insert(my_value_type
			("Crude_oil", -0.2));	
	ummp.insert(my_value_type
			("Gold", +0.1));	
	ummp.insert(my_value_type 
				("S&P", +0.5));	
	ummp.insert(my_value_type 
				("Crude_oil", +0.1));
	operators_of_unordered_multimap(ummp);
	
	multimap<string, double> mmp
			(ummp.begin(), ummp.end());
	operators_of_multimap(mmp);
	
	just_pause();
}


/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
