// RMSE_SD_answer.cpp

#include "my_essentials.hpp"

void rmse_sd(const vector<double>&, 
			const vector<double>&);

int main()
{
	array<double, 6> 
		act = {1.9, 2.8, 4.2, 4.4, 2.8, 1.7},
		pred= {2.0, 3.1, 3.9, 4.5, 3.2, 1.5};
	vector<double>actual(act.begin(), act.end());
	vector<double>prediction(pred.begin(),
					pred.end());
	rmse_sd(actual, prediction);
	just_pause();
}

void rmse_sd(const vector<double>& actual, 
		const vector<double>& prediction)
{	
	// (a) Difference:
	auto error = actual;

	transform(actual.begin(),
		actual.end(),
		prediction.begin(),
		error.begin(),
		minus<double>() );

	cout << "\n (a) The difference is:\n";
	cout_container("\terror", error);
	//--------------------------------------------

	// (b) RMSE:
	auto sum_square 
	= [](const double& sum, const double& x)
	->double{ return sum+x*x;};

	auto sum_sq_err = accumulate(error.begin(), 
			error.end(), 0.0, sum_square); 	// Note: instead of 0.0 if we use 0 then 
	// the accumulate would assume the numbers 
	// are integer instead of double.
	auto m_sq_err = sum_sq_err/error.size();
	auto rms_err = sqrt(m_sq_err);
	cout << "\n (b) The Root Mean Square Error "
		 <<  " (RMSE) = " << rms_err <<endl;
	//--------------------------------------------

	// (c) Std Dev.:
	// (i) Mean of differnce:

	auto mean = accumulate(error.begin(), 
				error.end(), 0.0);
	
	mean /= error.size();
	cout << "\n (c-i) The Mean of the Difference = " << mean <<endl;

	// (ii) RMS deviation:
	auto sum_square_dev 
	= [mean](const double& sum, const double& x)
		->double{
			static double dev(0.0);
			dev = x - mean;
			return sum+ dev*dev;
	  	};

	auto sum_sq_errdev = accumulate(
		error.begin(), error.end(), 
		0.0, sum_square_dev);

	auto m_sq_errdev = 
			sum_sq_errdev/error.size();
	auto std_dev = sqrt(m_sq_errdev);
	cout << "\n (c-ii) The Standard Deviation = "
					<< std_dev <<endl;

	// (iii) Proof of: RMSE^2 = SD^2 + MEAN^2
	auto lhs = rms_err* rms_err;
	auto rhs = std_dev*std_dev + mean*mean;
	cout 	<< "\n (c-iii) The square of RMSE  = "
		<< lhs	
		<< "\n\t The square of Std. Div.\n"
		<< "\t + square of Mean = "
		<< rhs << endl;

	cout<< "\n Hooray, we proved\n\t"
		<< "RMSE^2 = SD^2 + MEAN^2"
		<<"\n by using the STL algorithms!"
		<<endl;
}

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
