// funcPtr_function-wrapper_virtualfunction.cpp

#include "my_essentials.hpp"

void old_style_function_ptr(double, double, const vector<double>&);
void new_style_function(double, double, const vector<double>&);
void virtual_function(double, double, const vector<double>&);

int main()
{
	vector<double> year_days;
	year_days.reserve(4);
	//Let us assign days of 4 consecutive years:
	year_days.push_back(365);
	year_days.push_back(365);
	year_days.push_back(366);
	year_days.push_back(365);
	//Let us create two doubles:
	double a = 10.0;
	double b = 20.0;

	old_style_function_ptr(a, b, year_days);
	new_style_function(a, b, year_days);
	virtual_function(a, b, year_days);

	just_pause();
}

//..................................................

/* 
To demonstrate the runtime swithiching mechanisms, let us create few functions. 
So that, the mechanisms can switch over these functions.
*/

//Functions with double parameters:
double add(double a, double b) {return a+b;}
double sub(double a, double b) {return a-b;}
double mul(double a, double b) {return a*b;}
double divi(double a, double b) {return a/b;} 

//Function with single parameter:
double average(const vector<double>& vec)
{
	return ( 
	(accumulate(vec.begin(), vec.end(),0))
		/static_cast<double>(vec.size()) 
	);
}
//..................................................
// (1) The Old-style function pointer:
 
//Declaring old style function pointers:
double my_maths_2D (double a, double b, 
	double (* my_maths) (double, double) ) {
   	return (* my_maths)(a,b);
}	

double my_maths_1D (const vector<double> &a, 
	double (* my_maths) (const vector<double>&) ) {
   	return (* my_maths)(a);
}	

void old_style_function_ptr(double a, double b, const vector<double> &year_days)
{
	cout <<"\n(1) USING OLD STYLE "
			<< "\"function pointer\":"; 
	cout << "\n\n my_maths_2D("
			<<a<<", "<< b <<",  add) = " 
			<< my_maths_2D(a, b, add); 

	cout << "\n\n my_maths_2D("
			<<a<<", "<< b <<",  sub) = " 
			<< my_maths_2D(a, b, sub); 

	cout << "\n\n my_maths_2D("
			<<a<<", "<< b <<",  mul) = " 
			<< my_maths_2D(a, b, mul); 

	cout << "\n\n my_maths_2D("
			<<a<<", "<< b <<",  divi) = " 
			<< my_maths_2D(a, b, divi);
	
	cout << "\n\n my_maths_1D(year_days, average) = " 
			<< my_maths_1D(year_days, average);

	cout <<"\n\n----end of old style function pointer----\n";
}

//..................................................

// (2) The New-style function<> :
 
void new_style_function(double a, double b, const vector<double>& year_days)
{
	cout << "\n\n(2) USING NEW STYLE "
				<< "\"function<>\":";
	
	function<double(double,double)> my_maths_2D;
	cout << "\n\n Let, function<double(double,double)> my_maths_2D;";
	my_maths_2D = add;
	cout << "\n\n When my_maths_2D = add" <<endl
		<<"\tmy_maths_2D(" << a <<", "<< b 
		<< ") = " << my_maths_2D(a, b);
		
	my_maths_2D = sub;
	cout << "\n\n When my_maths_2D = sub" <<endl
		<<"\tmy_maths_2D(" << a <<", "<< b 			<< ") = " << my_maths_2D(a, b);

	my_maths_2D = mul;
	cout << "\n\n When my_maths_2D = mul" <<endl
		<<"\tmy_maths_2D(" << a <<", "<< b 			<< ") = " << my_maths_2D(a, b);

	my_maths_2D = divi;
	cout << "\n\n When my_maths_2D = divi" <<endl
		<<"\tmy_maths_2D(" << a <<", "<< b 
		<< ") = " << my_maths_2D(a, b);
	
	function<double(const vector<double>&)> my_maths_1D;
	cout << "\n\n Let, function"
	     << "<double(const vector<double>&)> "
	     << "my_maths_1D;";
	
	my_maths_1D = average;
	cout << "\n\n When my_maths_1D = "
	     <<"average" <<endl
	     <<"\tmy_maths_1D(year_days) = " 
	     << my_maths_1D(year_days) << endl;
	cout <<"\n----end of new style function<>----\n";
}	

//..................................................

// (3) The virtual function:

class base{
public:
	virtual double my_maths_2D(double, double){return 0;}
	virtual double my_maths_1D(const vector<double>&){return 0;}
};

class addition: public base {
public:
	double my_maths_2D(double a, double b){return add(a, b);}	
};

class subtraction: public base {
public:
	double my_maths_2D(double a, double b){return sub(a, b);}	
};

class multiplication: public base{
public:
	double my_maths_2D(double a, double b){return mul(a, b);}	
};

class divison: public base{
public:
	double my_maths_2D(double a, double b){return divi(a, b);}	
};

class average_data: public base{
public:
	double my_maths_1D(const vector<double>& a){return average(a);}	
};
	
void virtual_function(double a, double b, const vector<double>& year_days)
{	
	cout << "\n\n(3) USING VIRTUAL FUNCTIONS:\n";
	
	base *my_virtual;
	cout << "\n base *my_virtual;" << endl;
		
	addition my_add;
	my_virtual = &my_add;
	cout << "\n my_virtual = &my_add;\n"
		<< " my_virtual-> my_maths_2D(" 			<< a <<", "<<b<<") = "
		<< my_virtual-> my_maths_2D(a, b) 
		<< endl;
	
	subtraction my_sub;
	my_virtual = &my_sub;
	cout << "\n my_virtual = &my_sub;\n "
		<< "my_virtual-> my_maths_2D(" 
		<< a <<", "<<b<<") = " 
		<< my_virtual-> my_maths_2D(a, b) 
		<< endl;
	
	multiplication my_mul;
	my_virtual = &my_mul;
	cout << "\n my_virtual = &my_mul;\n "
		<< "my_virtual-> my_maths_2D(" 
		<< a <<", "<<b<<") = " 
		<< my_virtual-> my_maths_2D(a, b) 
		<< endl;
	
	divison my_divi;
	my_virtual = &my_divi;
	cout << "\n my_virtual = &my_divi;\n "
		<< "my_virtual-> my_maths_2D(" 
		<< a <<", "<<b<<") = " 
		<< my_virtual-> my_maths_2D(a, b) 
		<< endl;

	average_data my_average;
	my_virtual = &my_average;
	cout << "\n my_virtual = &my_average;\n "
		<< "my_virtual-> my_maths_1D("
		<< "year_days) = " 
		<< my_virtual-> my_maths_1D(year_days) 
		<< endl;
	cout <<"\n----end of virtual function----\n";
}

//..................................................

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
