// my_essentials.hpp

#ifndef MY_ESSENTIALS_H
#define MY_ESSENTIALS_H

#include <iostream>
#include <string>
#include <vector>
#include <list>
#include <deque>
#include <array>
#include <iterator>
#include <algorithm>
#include <numeric>
#include <functional>
#include <iomanip>
#include <fstream>
#include <exception>
#include <memory>
#include <random>

using namespace std;

//-----------------------------------------------

inline void just_pause(){
// the keyword "inline" requests the compiler to place 
// the object code of the function body at every point 
// in a program where the function is called.

	std::cin.clear();
	std::cout << "\nPlease enter a letter to exit...";
	char ch;
	std::cin >> ch;
}
//-----------------------------------------------

// Container independent code for printing out 
// container with one typename

template<typename container>
void cout_container(const std::string& message, const container& data)
{
	std::cout << message << "{";

	if(data.empty()){
		std::cout << "NULL}"<<std::endl;
		return;
	}

	auto start = data.begin();
	auto end = data.end();
	while(true)
	{
		std::cout << *start;
		++start;
		if(start != end)
			std::cout << ", ";
		else{
			std::cout <<'}' << std::endl;
			break;
		}
	}
}
//-----------------------------------------------

// Container independent code for printing out container 
// with two typenames

template<typename container>
inline void cout_container2T(
	const string& message, const container& data)
{
	cout << message << "{ ";

	if(data.empty())
	{
		cout << "NULL }"<<endl;
		return;
	}

	//*// Either using iterator
	auto start = data.begin();
	auto end = data.end();
	while( start != end)
	{
		cout << '(' <<start->first <<", "<<start->second<<") ";
		++start;
	}
	//*/
	/* OR using range-based for
	for(auto& i : data) // it suits for the VC12 and the GCC 4.7
		cout << '(' <<i.first <<", "<<i.second<<") ";
	*/

	cout <<"}"<<endl;
}
//------------------------------------------------

inline void sure_write(string& filename) 
{
	fstream fo(filename);
	bool flag1 = fo.is_open();
	fo.close();
	if(flag1)
	{
		cout << "\nThe file : "<< filename
		<< " already exists!\n"
		<<"Do you want to overwrite? (for yes"
		<<" enter y/for no enter n): ";
		char flag2;
		cin >> flag2;
		if(flag2 == 'n' || flag2 == 'N')
		{
			cout << "\n Ok, please enter "
				<<"another file name: ";
			cin >> filename;
			sure_write(filename);
		}
	}
}

//-----------------------------------------------

template<typename T, size_t columns>
inline void cout_array2d(const string& message, 
	const int& rows, const array<T,columns> data[])
{
	cout << message << "\n\t";
	if(columns == 0)
	{
		cout << "NULL "<<endl;
		return;
	}

	for(auto i = 0; i< rows; ++i)
	{
		for(auto j = 0; j < columns; ++j)
		{
			//cout << "(" << i << ", " << j <<endl;
			cout << data[i][j] << "  ";
		}
		cout << endl << '\t';
	}
}
//-----------------------------------------------

#endif

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
