// my_essentials_and_concurrency.hpp

#ifndef MY_ESSENTIALS_and_CONCURRENCY_H
#define MY_ESSENTIALS_and_CONCURRENCY_H

// General part:

#include <iostream>
#include <string>
#include <vector>
#include <list>
#include <deque>
#include <array>
#include <iterator>
#include <algorithm>
#include <numeric>
#include <functional>
#include <iomanip>
#include <fstream>
#include <exception>
#include <memory>
#include <random>

using namespace std;

//-----------------------------------------------

inline void just_pause(){
// the keyword "inline" requests the compiler to place 
// the object code of the function body at every point 
// in a program where the function is called.

	std::cin.clear();
	std::cout<<"\nPlease enter a letter to exit...";
	char ch;
	std::cin>>ch;
}
//-----------------------------------------------

// Container independent code for printing out 
// container with one typename

template<typename container>
void cout_container(const std::string& message, const container& data)
{
	std::cout << message << "{";

	if(data.empty()){
		std::cout << "NULL}"<<std::endl;
		return;
	}

	auto start = data.begin();
	auto end = data.end();
	while(true)
	{
		std::cout << *start;
		++start;
		if(start != end)
			std::cout << ", ";
		else{
			std::cout <<'}' << std::endl;
			break;
		}
	}
}
//-----------------------------------------------

// Container independent code for printing out container 
// with two typenames

template<typename container>
inline void cout_container2T(
	const string& message, const container& data)
{
	cout << message << "{ ";

	if(data.empty())
	{
		cout << "NULL }"<<endl;
		return;
	}

	//*// Either using iterator
	auto start = data.begin();
	auto end = data.end();
	while( start != end)
	{
		cout << '(' <<start->first <<", "<<start->second<<") ";
		++start;
	}
	//*/
	/* OR using range-based for
	for(auto& i : data) // it suits for the VC12 and the GCC 4.7
		cout << '(' <<i.first <<", "<<i.second<<") ";
	*/

	cout <<"}"<<endl;
}
//------------------------------------------------

inline void sure_write(string& filename) 
{
	fstream fo(filename);
	bool flag1 = fo.is_open();
	fo.close();
	if(flag1)
	{
		cout << "\nThe file : "<< filename
		<< " already exists!\n"
		<<"Do you want to overwrite? (for yes"
		<<" enter y/for no enter n): ";
		char flag2;
		cin >> flag2;
		if(flag2 == 'n' || flag2 == 'N')
		{
			cout << "\n Ok, please enter "
				<<"another file name: ";
			cin >> filename;
			sure_write(filename);
		}
	}
}
//-----------------------------------------------

template<typename T, size_t columns>
inline void cout_array2d(const string& message, 
	const int& rows, const array<T,columns> data[])
{
	cout << message << "\n\t";
	if(columns == 0)
	{
		cout << "NULL "<<endl;
		return;
	}

	for(auto i = 0; i< rows; ++i)
	{
		for(auto j = 0; j < columns; ++j)
		{
			//cout << "(" << i << ", " << j <<endl;
			cout << data[i][j] << "  ";
		}
		cout << endl << '\t';
	}


}
//-----------------------------------------------

// system_notepad:
//
// An execute system command of syntax:
// int system (const char * command);
// for opening  a given file in notepad
// in Microsoft Windows operating system:

auto system_notepad = [](string outfile){

	// if the operating system is NOT Windows
	// then just cout the name of the outputfile:
	#ifndef _MSC_VER
	cout << "\n\tThe output file: "
		 << outfile
		 << " is ready!\n\n";
	// else use notepad:
	#else

	string xl = "notepad " + outfile;

	char *cmd=new char[xl.size()+1];
	unsigned short sz = xl.size();
	for(auto i = 0; i < sz; ++i) {
		cmd[i] = xl[i];
	}
	cmd[sz] = 0; 
	// because, C-style character 
	// strings are null terminated.
	system(cmd);
	delete[] cmd;

	#endif
};
//------------------------------------------

// Concurrency related part:

// The following C++11 libraries 
// are not available in VC10, 
// but available in VC12.

#include <chrono>
#include <future>
#include <mutex>

//-----------------------------------------------

// time_now() and duration(t1, t2)

// (a) Lambda function to note present time:
auto time_now = []() {
	return chrono::system_clock::now();
};

// (b) Lambda function to estimate DURATION between
//     two time points in nanoseconds/microseconds/
//     milliseconds/seconds:
typedef chrono::system_clock::time_point tp;
auto duration = [](tp init, tp final){ return
		static_cast<double>(
	chrono::duration_cast<chrono::nanoseconds>
				(final-init).count()
		);
};
//-----------------------------------------------

// The Gaussian distribution based
// random number generator
// for financial calculations:

auto phi = []()->double{
	static double mean = 0.0;
	static double std_dev = 1.0;
	static mt19937 engine;
	static normal_distribution<double>
				normal(mean, std_dev);
	return normal(engine);
};
//-----------------------------------------------

#endif

/***************************************

Copyright  2013 Coding4You
N. S. Pattabi Raman 

Permission to use this software 
for academic or commercial purpose
is hereby granted to the programmers
who have purchased the book,
Concise exposure to C++.

N. S. Pattabi Raman makes no 
representations about the suitability
of this software for any purpose. 
It is provided "as is" without express 
or implied warranty. 

***************************************/
